@@ -19,7 +19,6 @@
 
 
 #include "playeritem.h"
-#include "characteritem.h"
 #include "player.h"
 #include "bonus.h"
 #include "bonusitem.h"
@@ -32,8 +31,6 @@
 #include <KStandardDirs>
 #include <KGameRenderer>
 
-static int nCounter = 0;
-
 const int PlayerItem::NB_FRAMES = 13;
 const int PlayerItem::ANIM_SPEED = 240;
 
@@ -54,6 +51,19 @@ PlayerItem::PlayerItem(Player* p_model, KGameRenderer* renderer) : CharacterItem
     m_animationTimer->setDuration(PlayerItem::ANIM_SPEED);
     connect(m_animationTimer, SIGNAL(frameChanged(int)), this, SLOT(setFrame(int)));
     
+    int width = Granatier::CellSize * 0.9;
+    int height = Granatier::CellSize * 0.9;
+    if(((int) Granatier::CellSize - width) % 2 != 0)
+    {
+        width--;
+    }
+    if(((int) Granatier::CellSize - height) % 2 != 0)
+    {
+        height--;
+    }
+    m_itemSizeSet = QSize(width, height);
+    m_itemSizeReal = m_itemSizeSet;
+    
     if(m_renderer->spriteExists("player_0"))
     {
         setSpriteKey("player_0");
@@ -62,7 +72,7 @@ PlayerItem::PlayerItem(Player* p_model, KGameRenderer* renderer) : CharacterItem
     setZValue(250);
     
     m_fallingAnimationCounter = 0;
-    m_ressurectionAnimationCounter = 0;
+    m_resurrectionAnimationCounter = 0;
 }
 
 PlayerItem::~PlayerItem()
@@ -75,35 +85,35 @@ void PlayerItem::resurrect()
     int nDirection = dynamic_cast <Player*> (m_model)->direction();
     setZValue(250);
     m_fallingAnimationCounter = 0;
-    m_ressurectionAnimationCounter = 10;
-    resetTransform();
+    m_resurrectionAnimationCounter = 10;
+    setRenderSize(m_renderSize);
     if(m_renderer->spriteExists("player_0"))
     {
         setSpriteKey("player_0");
     }
+    
     QTransform transform;
-    transform.translate(renderer()->boundsOnSprite(spriteKey()).width() / 2, renderer()->boundsOnSprite(spriteKey()).height() / 2);
+    transform.translate(m_itemSizeSet.width() / 2.0, m_itemSizeSet.height() / 2.0);
     // get the angle
     switch(nDirection)
     {
-        case Element::EAST:
+        case Granatier::Direction::EAST:
             transform.rotate(0);
             break;
-        case Element::SOUTH:
+        case Granatier::Direction::SOUTH:
             transform.rotate(90);
             break;
-        case Element::NORTH:
+        case Granatier::Direction::NORTH:
             transform.rotate(-90);
             break;
-        case Element::WEST:
+        case Granatier::Direction::WEST:
             transform.rotate(180);
             break;
         default:
             transform.rotate(0);
             break;
     }
-    transform.scale(1, 1);
-    transform.translate(-renderer()->boundsOnSprite(spriteKey()).width() / 2, -renderer()->boundsOnSprite(spriteKey()).height() / 2);
+    transform.translate(-m_itemSizeReal.width() / 2.0, -m_itemSizeReal.height() / 2.0);
     setTransform(transform);
     
     startAnim();
@@ -114,32 +124,39 @@ void PlayerItem::resurrect()
 void PlayerItem::updateDirection()
 {
     int nDirection = dynamic_cast <Player*> (m_model)->direction();
+    
     // Rotate the item
     QTransform transform;
-    transform.translate(renderer()->boundsOnSprite(spriteKey()).width() / 2, renderer()->boundsOnSprite(spriteKey()).height() / 2);
+    transform.translate(m_itemSizeSet.width() / 2.0, m_itemSizeSet.height() / 2.0);
     // get the angle
     switch(nDirection)
     {
-        case Element::EAST:
+        case Granatier::Direction::EAST:
             transform.rotate(0);
             break;
-        case Element::SOUTH:
+        case Granatier::Direction::SOUTH:
             transform.rotate(90);
             break;
-        case Element::NORTH:
+        case Granatier::Direction::NORTH:
             transform.rotate(-90);
             break;
-        case Element::WEST:
+        case Granatier::Direction::WEST:
             transform.rotate(180);
             break;
         default:
             transform.rotate(0);
             break;
     }
-    transform.translate(-renderer()->boundsOnSprite(spriteKey()).width() / 2, -renderer()->boundsOnSprite(spriteKey()).height() / 2);
+    transform.translate(-m_itemSizeReal.width() / 2.0, -m_itemSizeReal.height() / 2.0);
     setTransform(transform);
 }
 
+void PlayerItem::updateGraphics(qreal svgScaleFactor)
+{
+    updateGraphicsInternal(svgScaleFactor);
+    updateDirection();
+}
+
 void PlayerItem::manageCollision()
 {
     QList<QGraphicsItem*> collidingList = collidingItems();
@@ -228,7 +245,7 @@ void PlayerItem::stopAnim()
     {
         setSpriteKey("player_0");
     }
-    if (m_animationTimer->state() != QTimeLine::NotRunning && m_ressurectionAnimationCounter == 0)
+    if (m_animationTimer->state() != QTimeLine::NotRunning && m_resurrectionAnimationCounter == 0)
     {
         m_animationTimer->stop();
     }
@@ -245,7 +262,7 @@ void PlayerItem::setFrame(const int p_frame)
     {
         setSpriteKey(QString("player_%1").arg(p_frame));
         
-        if(m_fallingAnimationCounter > 0 || m_ressurectionAnimationCounter > 0)
+        if(m_fallingAnimationCounter > 0 || m_resurrectionAnimationCounter > 0)
         {
             int angle = 0;
             int nDirection = dynamic_cast <Player*> (m_model)->direction();
@@ -253,16 +270,16 @@ void PlayerItem::setFrame(const int p_frame)
             // get the angle
             switch(nDirection)
             {
-                case Element::EAST:
+                case Granatier::Direction::EAST:
                     angle = 0;
                     break;
-                case Element::SOUTH:
+                case Granatier::Direction::SOUTH:
                     angle = 90;
                     break;
-                case Element::NORTH:
+                case Granatier::Direction::NORTH:
                     angle = -90;
                     break;
-                case Element::WEST:
+                case Granatier::Direction::WEST:
                     angle = 180;
                     break;
                 default:
@@ -276,10 +293,10 @@ void PlayerItem::setFrame(const int p_frame)
                 setZValue(-2);
                 // shrink the item
                 QTransform transform;
-                transform.translate(renderer()->boundsOnSprite(spriteKey()).width() / 2, renderer()->boundsOnSprite(spriteKey()).height() / 2);
+                transform.translate(m_itemSizeSet.width() / 2.0, m_itemSizeSet.width() / 2.0);
                 transform.rotate(angle);
-                transform.scale(1-m_fallingAnimationCounter*0.02, 1-m_fallingAnimationCounter*0.02);
-                transform.translate(-renderer()->boundsOnSprite(spriteKey()).width() / 2, -renderer()->boundsOnSprite(spriteKey()).height() / 2);
+                setRenderSize(m_renderSize * (1-m_fallingAnimationCounter*0.02));
+                transform.translate(-m_itemSizeReal.width() * (1-m_fallingAnimationCounter*0.02) / 2.0, -m_itemSizeReal.width() * (1-m_fallingAnimationCounter*0.02) / 2.0);
                 setTransform(transform);
                 m_fallingAnimationCounter++;
                 
@@ -291,60 +308,62 @@ void PlayerItem::setFrame(const int p_frame)
                 }
             }
             
-            if(m_ressurectionAnimationCounter > 0)
+            if(m_resurrectionAnimationCounter > 0)
             {
+                qreal resurrectionScale = 1;
                 QTransform transform;
-                transform.translate(renderer()->boundsOnSprite(spriteKey()).width() / 2, renderer()->boundsOnSprite(spriteKey()).height() / 2);
+                transform.translate(m_itemSizeSet.width() / 2.0, m_itemSizeSet.width() / 2.0);
                 transform.rotate(angle);
-                if(m_ressurectionAnimationCounter > 9)
+                if(m_resurrectionAnimationCounter > 9)
                 {
-                    transform.scale(1.1, 1.1);
+                    resurrectionScale = 1.1;
                 }
-                else if(m_ressurectionAnimationCounter > 8)
+                else if(m_resurrectionAnimationCounter > 8)
                 {
-                    transform.scale(0.9, 0.9);
+                    resurrectionScale = 0.9;
                 }
-                else if(m_ressurectionAnimationCounter > 7)
+                else if(m_resurrectionAnimationCounter > 7)
                 {
-                    transform.scale(0.6, 0.6);
+                    resurrectionScale = 0.6;
                 }
-                else if(m_ressurectionAnimationCounter > 6)
+                else if(m_resurrectionAnimationCounter > 6)
                 {
-                    transform.scale(0.7, 0.7);
+                    resurrectionScale = 0.7;
                 }
-                else if(m_ressurectionAnimationCounter > 5)
+                else if(m_resurrectionAnimationCounter > 5)
                 {
-                    transform.scale(0.85, 0.85);
+                    resurrectionScale = 0.85;
                 }
-                else if(m_ressurectionAnimationCounter > 4)
+                else if(m_resurrectionAnimationCounter > 4)
                 {
-                    transform.scale(1.0, 1.0);
+                    resurrectionScale = 1;
                 }
-                else if(m_ressurectionAnimationCounter > 3)
+                else if(m_resurrectionAnimationCounter > 3)
                 {
-                    transform.scale(1.05, 1.05);
+                    resurrectionScale = 1.05;
                 }
-                else if(m_ressurectionAnimationCounter > 2)
+                else if(m_resurrectionAnimationCounter > 2)
                 {
-                    transform.scale(1.1, 1.1);
+                    resurrectionScale = 1.1;
                 }
-                else if(m_ressurectionAnimationCounter > 1)
+                else if(m_resurrectionAnimationCounter > 1)
                 {
-                    transform.scale(1.05, 1.05);
+                    resurrectionScale = 1.05;
                 }
-                else if(m_ressurectionAnimationCounter > 0)
+                else if(m_resurrectionAnimationCounter > 0)
                 {
-                    transform.scale(1.0, 1.0);
+                    resurrectionScale = 1;
                 }
                 
-                m_ressurectionAnimationCounter--;
-                if(m_ressurectionAnimationCounter == 0)
+                m_resurrectionAnimationCounter--;
+                if(m_resurrectionAnimationCounter == 0)
                 {
-                    transform.scale(1.0, 1.0);
+                    resurrectionScale = 1;
                     stopAnim();
                 }
                 
-                transform.translate(-renderer()->boundsOnSprite(spriteKey()).width() / 2, -renderer()->boundsOnSprite(spriteKey()).height() / 2);
+                setRenderSize(m_renderSize * resurrectionScale);
+                transform.translate(-m_itemSizeReal.width() * resurrectionScale / 2.0, -m_itemSizeReal.width() * resurrectionScale / 2.0);
                 setTransform(transform);
             }
         }
@@ -355,9 +374,13 @@ void PlayerItem::setDead()
 {
     stopAnim();
     setZValue(1);
-    if(m_ressurectionAnimationCounter != 0)
+    if(m_resurrectionAnimationCounter != 0)
     {
-        resetTransform();
+        QTransform transform;
+        transform.translate(m_itemSizeSet.width() / 2.0, m_itemSizeSet.height() / 2.0);
+        setRenderSize(m_renderSize);
+        transform.translate(-m_itemSizeReal.width() / 2.0, -m_itemSizeReal.height() / 2.0);
+        setTransform(transform);
     }
     if(m_renderer->spriteExists("player_death"))
     {
