@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,9 +30,9 @@ from meld import Meld
 from animation import Animation, Animated, animate
 from message import Message
 
-from util import logException, m18nc, isAlive, kprint
+from util import logDebug, logException, m18nc, isAlive, kprint, stack, uniqueList
 import common
-from common import elements, WINDS, LIGHTSOURCES, InternalParameters, ZValues
+from common import elements, WINDS, LIGHTSOURCES, InternalParameters, ZValues, Debug
 
 ROUNDWINDCOLOR = QColor(235, 235, 173)
 
@@ -202,6 +202,7 @@ class Board(QGraphicsRectItem):
             return self._focusTile
         def fset(self, tile):
             # pylint: disable=W0212
+            prevTile = self._focusTile
             if tile:
                 assert tile.element != 'Xy', tile
                 if not isinstance(tile.board, DiscardBoard):
@@ -209,6 +210,13 @@ class Board(QGraphicsRectItem):
                 self._focusTile = tile
             else:
                 self._focusTile = self.autoSelectTile()
+            if self._focusTile and self._focusTile.element in Debug.focusable:
+                logDebug('new focus tile %s from %s' % (self._focusTile.element, stack('')[-1]))
+            if (self._focusTile != prevTile
+                and self.isHandBoard and self.player
+                and not self.player.game.isScoringGame()
+                and InternalParameters.field.clientDialog):
+                InternalParameters.field.clientDialog.focusTileChanged()
             if self.hasFocus:
                 self.scene().focusBoard = self
         return property(**locals())
@@ -274,10 +282,15 @@ class Board(QGraphicsRectItem):
     def __moveCursor(self, key):
         """move focus"""
         tiles = self._focusableTiles(key)
-        oldPos = self.focusTile.xoffset, self.focusTile.yoffset
-        tiles = list(x for x in tiles if (x.xoffset, x.yoffset) != oldPos or x == self.focusTile)
-        tiles.append(tiles[0])
-        self.focusTile = tiles[tiles.index(self.focusTile)+1]
+        if tiles:
+            # sometimes the handBoard still has focus but
+            # has no focusable tiles. Like after declaring
+            # Original Call.
+            oldPos = self.focusTile.xoffset, self.focusTile.yoffset
+            tiles = list(x for x in tiles if (x.xoffset, x.yoffset) != oldPos or x == self.focusTile)
+            assert tiles, [str(x) for x in self.tiles]
+            tiles.append(tiles[0])
+            self.focusTile = tiles[tiles.index(self.focusTile)+1]
 
     def dragObject(self, tile):
         """returns the object that should be dragged when the user tries to drag
@@ -332,7 +345,7 @@ class Board(QGraphicsRectItem):
         matrix = (int(matrix.m11()), int(matrix.m12()), int(matrix.m21()), int(matrix.m22()))
         rotations = {(0, 0, 0, 0):0, (1, 0, 0, 1):0, (0, 1, -1, 0):90, (-1, 0, 0, -1):180, (0, -1, 1, 0):270}
         if matrix not in rotations:
-            raise Exception('matrix unknown:%s' % matrix)
+            raise Exception('matrix unknown:%s' % str(matrix))
         return rotations[matrix]
 
     def setPos(self, xWidth=0, xHeight=0, yWidth=0, yHeight=0):
@@ -355,7 +368,7 @@ class Board(QGraphicsRectItem):
         sizeX = self.tileset.faceSize.width() * self.__fixedWidth
         sizeY = self.tileset.faceSize.height() * self.__fixedHeight
         if self.showShadows:
-            sizeX += self.tileset.shadowWidth()
+            sizeX += self.tileset.shadowWidth() + 2 * self.tileset.shadowHeight()
             sizeY += self.tileset.shadowHeight()
         rect = self.rect()
         rect.setWidth(sizeX)
@@ -379,8 +392,10 @@ class Board(QGraphicsRectItem):
         This is also called when the tileset or the light source for this board changes"""
         width = self.tileset.faceSize.width()
         height = self.tileset.faceSize.height()
-        if self.isHandBoard or not self.showShadows:
+        if not self.showShadows:
             offsets = (0, 0)
+        elif self.isHandBoard:
+            offsets = (-self.tileset.shadowHeight() * 2, 0)
         else:
             offsets = self.tileset.shadowOffsets(self._lightSource, self.sceneRotation())
         newX = self.xWidth*width+self.xHeight*height + offsets[0]
@@ -440,6 +455,7 @@ class Board(QGraphicsRectItem):
             self._tileset = tileset
             self._lightSource = lightSource
             self._showShadows = showShadows
+            self.setGeometry()
             for child in self.childItems():
                 if isinstance(child, (Board, PlayerWind)):
                     child.tileset = tileset
@@ -449,7 +465,6 @@ class Board(QGraphicsRectItem):
                 self.placeTile(tile)
                 tile.graphics.update()
             self.computeRect()
-            self.setGeometry()
             if self.hasFocus:
                 self.scene().focusBoard = self
 
@@ -499,6 +514,7 @@ class Board(QGraphicsRectItem):
         shiftZ = self.shiftZ(tile.level)
         boardPos = QPointF(tile.xoffset*width, tile.yoffset*height) + shiftZ
         scenePos = self.mapToScene(boardPos)
+        tile.graphics.setDrawingOrder()
         return {'pos': scenePos, 'rotation': self.sceneRotation(), 'scale': self.scale()}
 
     def placeTile(self, tile):
@@ -523,30 +539,24 @@ class Board(QGraphicsRectItem):
 
 class CourtBoard(Board):
     """A Board that is displayed within the wall"""
-
     def __init__(self, width, height):
         Board.__init__(self, width, height, InternalParameters.field.tileset)
 
     def maximize(self):
-        """make it as big as possible within the wall. This code is inefficient...
-        but fast enough. When resizing, recomputing the SVG
-         tiles takes much more time than this."""
-        xWidth = 1.5
-        yWidth = 1.5
+        """make it as big as possible within the wall"""
         cWall = InternalParameters.field.game.wall
-        while self.collidesWithItem(cWall[3]):
-            xWidth += 0.01
-            self.setPos(xWidth=xWidth, yWidth=yWidth)
-        while self.collidesWithItem(cWall[2]):
-            yWidth += 0.01
-            self.setPos(xWidth=xWidth, yWidth=yWidth)
-        scaleFactor = 2.0
-        Board.setScale(self, scaleFactor)
-        while self.collidesWithItem(cWall[0]) or \
-            self.collidesWithItem(cWall[1]):
-            scaleFactor *= 0.99
-            self.resetTransform()
-            Board.setScale(self, scaleFactor)
+        newSceneX = cWall[3].sceneBoundingRect().right()
+        newSceneY = cWall[2].sceneBoundingRect().bottom()
+        QGraphicsRectItem.setPos(self, newSceneX, newSceneY)
+        tileset = self.tileset
+        avail = (cWall[2].sceneBoundingRect().width()
+            - 2 * tileset.shadowHeight()
+            - tileset.faceSize.height())
+        xScaleFactor = avail / (self.width * tileset.faceSize.width() + tileset.shadowWidth())
+        yScaleFactor = avail / (self.height * tileset.faceSize.height() + tileset.shadowHeight())
+        Board.setScale(self, min(xScaleFactor, yScaleFactor))
+        for tile in self.tiles:
+            tile.board.placeTile(tile)
 
 class SelectorBoard(CourtBoard):
     """a board containing all possible tiles for selection"""
@@ -585,7 +595,7 @@ class SelectorBoard(CourtBoard):
     def dropEvent(self, event):
         """drop a tile into the selector"""
         mime = event.mimeData()
-        self.receive(mime.tile, mime.meld)
+        self.dropHere(mime.tile, mime.meld)
         event.accept()
 
     def receive(self, tile=None, meld=None):
@@ -601,7 +611,7 @@ class SelectorBoard(CourtBoard):
         (senderHand if senderHand.tiles else self).hasFocus = True
         self._noPen()
 
-    def dropHere(self, tile, meld, dummyLowerHalf):
+    def dropHere(self, tile, meld, dummyLowerHalf=None):
         """drop tile or meld into selector board"""
         tile1 = tile or meld[0]
         if tile1.board != self:
@@ -708,8 +718,13 @@ class FittingView(QGraphicsView):
         self.dragObject = None
         self.setFocus()
 
-    def wheelEvent(self, dummyEvent):
-        """we do not want scrolling for the scene view"""
+    def wheelEvent(self, event):  # pylint: disable=R0201
+        """we do not want scrolling for the scene view.
+        Instead scrolling down changes perspective like in kmahjongg"""
+        if event.orientation() == Qt.Vertical and event.delta() < 0:
+            InternalParameters.field.changeAngle()
+        # otherwise do not call ignore() because we do want
+        # to consume this
 
     def resizeEvent(self, dummyEvent):
         """scale the scene for new view size"""
@@ -737,27 +752,30 @@ class FittingView(QGraphicsView):
         return item.tile.board.tileFaceRect().contains(itemPos)
 
     def tileAt(self, position):
-        """find out which tile is clickable at this position"""
+        """find out which tile is clickable at this position. Always
+        returns a list. If there are several tiles above each other,
+        return all of them, highest first"""
         allTiles = [x for x in self.items(position) if isinstance(x, GraphicsTileItem)]
         items = [x for x in allTiles if self.__matchingTile(position, x)]
         if not items:
             return None
         items = [x.tile for x in items]
-        maxLevel = max(x.level for x in items)
-        item = [x for x in items if x.level == maxLevel][0]
-        for other in [x.tile for x in allTiles]:
-            if (other.xoffset, other.yoffset) == (item.xoffset, item.yoffset):
-                if other.level > item.level:
-                    item = other
-        return item
+        for item in items[:]:
+            for other in [x.tile for x in allTiles]:
+                if (other.xoffset, other.yoffset) == (item.xoffset, item.yoffset):
+                    if other.level > item.level:
+                        items.append(other)
+        return uniqueList(sorted(items, key=lambda x: -x.level))
 
     def mousePressEvent(self, event):
         """set blue focus frame"""
         board = InternalParameters.field.discardBoard
-        tile = self.tileAt(event.pos())
-        if tile:
+        tiles = self.tileAt(event.pos())
+        if tiles:
             if event.modifiers() & Qt.ShiftModifier:
-                kprint(tile)
+                for tile in tiles:
+                    kprint('%s: board.level:%s' % (str(tile), tile.board.level))
+            tile = tiles[0]
             board = tile.board
             isRemote = board.isHandBoard and board.player and not board.player.game.isScoringGame()
             if board.isHandBoard and not isRemote:
@@ -816,19 +834,18 @@ class YellowText(QGraphicsRectItem):
         QGraphicsRectItem.__init__(self, side)
         self.side = side
         self.font = QFont()
-        self.font.setWeight(QFont.Bold)
-        self.font.setPointSize(36)
-        self.height = 50
+        self.font.setPointSize(48)
+        self.height = 62
         self.width = 200
         self.msg = None
         self.setText('')
 
     def setText(self, msg):
         """set the text of self"""
-        self.msg = msg
+        self.msg = '%s  ' % msg
         metrics = QFontMetrics(self.font)
-        self.width = metrics.width(msg)
-        self.height = metrics.height()
+        self.width = metrics.width(self.msg)
+        self.height = metrics.lineSpacing() * 1.1
         self.setRect(0, 0, self.width, self.height)
         self.resetTransform()
         rotation = self.side.rotation()
@@ -837,7 +854,10 @@ class YellowText(QGraphicsRectItem):
             yOffset = self.rect().height()
             if rotation == 0:
                 yOffset = 2 * -yOffset
-            self.translate(-self.rect().width()/2, yOffset)
+            if rotation == 180:
+                self.translate(self.rect().width()/2, yOffset)
+            else:
+                self.translate(-self.rect().width()/2, yOffset)
         else:
             self.translate(-self.rect().width()/2, -self.rect().height()/2)
     def paint(self, painter, dummyOption, dummyWidget):
