@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -31,29 +31,78 @@ class Debug:
     """holds flags for debugging output. At a later time we might
     want to add command line parameters for initialisation, and
     look at kdebugdialog"""
-    animation = [] # ['ye', 'ys', 'yw', 'yn', 'fe', 'fs', 'fw', 'fn']
+    traffic = False
+    sql = False
+    animation = '' # 'yeysywynfefsfwfn'
     animationSpeed = False
     robotAI = False
     dangerousGame = False
+    originalCall = False
     modelTest = False
-    regex = False
-    profileRegex = False
+    focusable = ''
+    robbingKong = False
+    mahJongg = False
+    sound = False
+    chat = False
+    argString = None
+    scores = False
+    handCache = False
+    explain = False
+    random = False
 
     def __init__(self):
         raise Exception('Debug is not meant to be instantiated')
 
+    @staticmethod
+    def help():
+        """a string for help texts about debug options"""
+        def optYielder(options):
+            """yields options with markers for line separation"""
+            for idx, opt in enumerate(options):
+                yield opt
+                if idx < len(options) - 1 and idx % 5 == 4:
+                    yield 'SEPARATOR'
+        options = list(x for x in Debug.__dict__ if not x.startswith('_'))
+        boolOptions = list(x for x in options if isinstance(Debug.__dict__[x], bool))
+        stringOptions = list(x for x in options if isinstance(Debug.__dict__[x], basestring))
+        stringExample = '%s=%s' % (stringOptions[0], 's3s4')
+        allOptions = boolOptions + stringOptions
+        opt = '\n'.join(', '.join(optYielder(allOptions)).split(' SEPARATOR, '))
+        return """set debug options. Pass a comma separated list of options.
+Options are: {opt}.
+Options {stropt} take a string argument like {example}""".format(
+           opt=opt,
+           stropt=', '.join(stringOptions), example=stringExample)
+
+    @staticmethod
+    def setOptions(args):
+        """args comes from the command line. Put this in the Debug class.
+        If something goes wrong, return an error message."""
+        if not args:
+            return
+        Debug.argString = args
+        for arg in args.split(','):
+            parts = arg.split('=')
+            if len(parts) == 1:
+                parts.append(True)
+            option, value = parts
+            if option not in Debug.__dict__:
+                return '--debug: unknown option %s' % option
+            if type(Debug.__dict__[option]) != type(value):
+                return '--debug: wrong value for option %s' % option
+            Debug.__dict__[option] = value
+
 class InternalParameters:
     """they are never saved in a config file. Some of them
     can be defined on the command line."""
+    version = '4.9.0'
     scaleScene = True
     reactor = None
-    seed = None
+    game = None # will only be set by command line --game
     autoPlay = False
     showRulesets = False
     autoPlayRulesetName = None	# will only be set by command line --autoplay
     autoPlayRuleset = None	# will only be set by command line --autoplay
-    showSql = False
-    showTraffic = False
     dbPath = None
     app = None
     socket = None
@@ -61,11 +110,20 @@ class InternalParameters:
     field = None
     hasGUI = False
     isServer = False
-    AI = 'default'
+    AI = 'Default'
+    csv = None
+    logPrefix = 'C'
+    continueServer = False
 
     def __init__(self):
         raise Exception('InternalParameters is not meant to be instantiated')
 
+    @staticmethod
+    def defaultPort():
+        """8000 plus version: for version 4.9.5 we use 8409"""
+        parts = InternalParameters.version.split('.')
+        return 8000 + int(parts[0]) * 100 + int(parts[1])
+
 class IntDict(defaultdict):
     """a dict where the values are expected to be numeric, so
     we can add dicts.If parent is given, parent is expected to
@@ -81,7 +139,7 @@ class IntDict(defaultdict):
         """need to reimplement this because the __init__ signature of
         IntDict is not identical to that of defaultdict"""
         result = IntDict(self.parent)
-        defaultdict.update(self, self)
+        defaultdict.update(result, self)
         # see http://www.logilab.org/ticket/23986
         return result
 
@@ -114,7 +172,7 @@ class IntDict(defaultdict):
         """how many tiles defined by countFilter do we hold?
         countFilter is an iterator of element names. No countFilter: Take all
         So count(['we','ws']) should return 8"""
-        return sum(self[x] for x in countFilter or self)
+        return sum((defaultdict.get(self, x) or 0) for x in countFilter or self)
 
     def all(self, countFilter=None):
         """returns a list of all tiles defined by countFilter, each tile multiplied by its occurrence
@@ -156,14 +214,22 @@ class IntDict(defaultdict):
 
 class Elements(object):
     """represents all elements"""
+    # pylint: disable=R0902
+    # too many attributes
     def __init__(self):
         self.occurrence = IntDict() # key: db, s3 etc. value: occurrence
         self.winds = set(['we', 'ws', 'ww', 'wn'])
+        self.wINDS = set(['We', 'Ws', 'Ww', 'Wn'])
         self.dragons = set(['db', 'dg', 'dr'])
+        self.dRAGONS = set(['Db', 'Dg', 'Dr'])
         self.honors = self.winds | self.dragons
+        self.hONORS = self.wINDS | self.dRAGONS
         self.terminals = set(['s1', 's9', 'b1', 'b9', 'c1', 'c9'])
+        self.tERMINALS = set(['S1', 'S9', 'B1', 'B9', 'C1', 'C9'])
         self.majors = self.honors | self.terminals
+        self.mAJORS = self.hONORS | self.tERMINALS
         self.minors = set()
+        self.mINORS = set()
         self.greenHandTiles = set(['dg', 'b2', 'b3', 'b4', 'b6', 'b8'])
         for color in 'sbc':
             for value in '2345678':
