@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,114 +21,189 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 import datetime
 from random import Random
 from collections import defaultdict
-
-from util import logError, logException, logDebug, m18n, isAlive
+from twisted.internet.defer import succeed
+from util import logError, logException, logDebug, m18n, isAlive, stack
 from common import WINDS, InternalParameters, elements, IntDict, Debug
 from query import Transaction, Query
-from scoringengine import Ruleset
+from rule import Ruleset
 from tile import Tile
-from scoringengine import HandContent
+from meld import tileKey
+from hand import Hand
 from sound import Voice
 from wall import Wall
 from move import Move
 from animation import Animated
 from player import Player, Players
 
+class CountingRandom(Random):
+    """counts how often random() is called and prints debug info"""
+    def __init__(self, game, value=None):
+        self.game = game
+        Random.__init__(self, value)
+        self.count = 0
+    def random(self):
+        """the central randomizator"""
+        self.count += 1
+        return Random.random(self)
+    def seed(self, newSeed=None):
+        self.count = 0
+        if Debug.random:
+            self.game.debug('Random gets seed %s' % newSeed)
+        Random.seed(self, newSeed)
+    def shuffle(self, listValue, func=None, intType=int):
+        oldCount = self.count
+        Random.shuffle(self, listValue, func, intType)
+        if Debug.random:
+            self.game.debug('%d calls to random by Random.shuffle from %s' % (
+                self.count - oldCount, stack('')[-2]))
+    def randrange(self, start, stop=None, step=1, intType=int, default=None, maxWidth=9007199254740992L):
+        oldCount = self.count
+        result = Random.randrange(self, start, stop, step, intType, default, maxWidth)
+        if Debug.random:
+            self.game.debug('%d calls to random by Random.randrange(%d,%s) from %s' % (
+                self.count - oldCount, start, stop, stack('')[-2]))
+        return result
+    def choice(self, fromList):
+        if len(fromList) == 1:
+            return fromList[0]
+        oldCount = self.count
+        result = Random.choice(self, fromList)
+        if Debug.random:
+            self.game.debug('%d calls to random by Random.choice(%s) from %s' % (
+                self.count - oldCount, str([str(x) for x in fromList]), stack('')[-2]))
+        return result
+    def sample(self, population, wantedLength):
+        oldCount = self.count
+        result = Random.sample(self, population, wantedLength)
+        if Debug.random:
+            self.game.debug('%d calls to random by Random.sample(x, %d) from %s' % (
+                self.count - oldCount, wantedLength, stack('')[-2]))
+        return result
+
 class Game(object):
     """the game without GUI"""
     # pylint: disable=R0902
     # pylint we need more than 10 instance attributes
 
-    def __init__(self, names, ruleset, gameid=None, seed=None, shouldSave=True, client=None):
+    def __init__(self, names, ruleset, gameid=None, wantedGame=None, shouldSave=True, client=None):
         """a new game instance. May be shown on a field, comes from database if gameid is set
 
         Game.lastDiscard is the tile last discarded by any player. It is reset to None when a
-        player gets a tile from the living end of the wall.
+        player gets a tile from the living end of the wall or after he claimed a discard.
         """
         # pylint: disable=R0915
         # pylint we need more than 50 statements
-        field = InternalParameters.field
-        if field:
-            field.game = self
-        self.randomGenerator = Random()
+        self.players = Players() # if we fail later on in init, at least we can still close the program
         self.client = client
-        self.seed = None
-        if not self.isScoringGame():
-            self.seed = seed or InternalParameters.seed or int(self.randomGenerator.random() * 10**12)
-        self.shouldSave = shouldSave
-        self.randomGenerator.seed(self.seed)
         self.rotated = 0
-        self.players = [] # if we fail later on in init, at least we can still close the program
-        self.activePlayer = None
+        self.notRotated = 0 # counts hands since last rotation
         self.ruleset = None
+        self.roundsFinished = 0
+        self.seed = 0
+        self.randomGenerator = CountingRandom(self)
+        if self.isScoringGame():
+            self.wantedGame = str(wantedGame)
+            self.seed = wantedGame
+        else:
+            self.wantedGame = wantedGame
+            _ = int(wantedGame.split('/')[0]) if wantedGame else 0
+            self.seed = _ or int(self.randomGenerator.random() * 10**9)
+        self.shouldSave = shouldSave
+        self.setHandSeed()
+        self.activePlayer = None
         self.winner = None
         self.moves = []
-        self.roundsFinished = 0
         self.myself = None   # the player using this client instance for talking to the server
         self.gameid = gameid
-        self.setGameId()
         self.playOpen = False
         self.autoPlay = False
         self.handctr = 0
+        self.roundHandCount = 0
+        self.handDiscardCount = 0
         self.divideAt = None
         self.lastDiscard = None # always uppercase
         self.visibleTiles = IntDict()
         self.discardedTiles = IntDict(self.visibleTiles) # tile names are always lowercase
         self.eastMJCount = 0
-        self.dangerousTiles = set()
+        self.dangerousTiles = list()
+        self.csvTags = []
+        self.setGameId()
         self.__useRuleset(ruleset)
         # shift rules taken from the OEMC 2005 rules
         # 2nd round: S and W shift, E and N shift
         self.shiftRules = 'SWEN,SE,WE'
+        field = InternalParameters.field
         if field:
+            field.game = self
+            field.startingGame = False
+            field.updateGUI()
             field.showWall()
         else:
             self.wall = Wall(self)
-        for name in names:
-            Players.createIfUnknown(name)
-        if field:
-            self.players = field.genPlayers()
-        else:
-            self.players = Players([Player(self) for idx in range(4)])
-        for idx, player in enumerate(self.players):
-            player.name = names[idx]
-            player.wind = WINDS[idx]
-        if self.client and self.client.username:
-            self.myself = self.players.byName(self.client.username)
+        self.assignPlayers(names)
+        if self.belongsToGameServer():
+            self.shufflePlayers()
+        if not self.isScoringGame() and '/' in self.wantedGame:
+            part = self.wantedGame.split('/')[1]
+            roundsFinished = 'ESWN'.index(part[0])
+            for _ in range(roundsFinished * 4 + int(part[1]) - 1):
+                self.rotateWinds()
+            for char in part[2:]:
+                self.notRotated += self.notRotated * 26 + ord(char) + 1 - ord('a')
         if self.shouldSave:
             self.saveNewGame()
         if field:
             self.initVisiblePlayers()
-            field.refresh()
+            field.updateGUI()
             self.wall.decorate()
 
+    def addCsvTag(self, tag, forAllPlayers=False):
+        """tag will be written to tag field in csv row"""
+        if forAllPlayers or self.belongsToHumanPlayer():
+            self.csvTags.append('%s/%s' % (tag, self.handId()))
+
+    def isFirstHand(self):
+        """as the name says"""
+        return self.roundHandCount == 0 and self.roundsFinished == 0
+
+    def handId(self, withAI=True):
+        """identifies the hand for window title and scoring table"""
+        aiVariant = ''
+        if withAI and self.belongsToHumanPlayer():
+            aiName = self.client.intelligence.name()
+            if aiName != 'Default':
+                aiVariant = aiName + '/'
+        num = self.notRotated
+        charId = ''
+        while num:
+            charId = chr(ord('a') + (num-1) % 26) + charId
+            num = (num-1) / 26
+        if self.finished():
+            wind = 'X'
+        else:
+            wind = WINDS[self.roundsFinished]
+        return '%s%s/%s%s%s' % (aiVariant, self.seed, wind, self.rotated + 1, charId)
+
     def setGameId(self):
         """virtual"""
         assert not self # we want it to fail, and quiten pylint
 
-    def close(self, dummyCallback=None):
-        """log off from the server"""
-        self.hide()
+    def close(self):
+        """log off from the server and return a Deferred"""
+        InternalParameters.autoPlay = False # do that only for the first game
+        deferred = succeed(None)
+        if self.client:
+            if self.client.perspective:
+                deferred = self.client.logout()
+            self.client = None
+        return deferred
 
-    def hide(self, dummyResult=None):
-        """if the game is shown in the client, hide it"""
-        field = InternalParameters.field
-        if field and isAlive(field):
-            for player in self.players:
-                if player.handBoard:
-                    player.clearHand()
-                    player.handBoard.hide()
-                    player.handBoard = None
-            field.setWindowTitle('Kajongg')
-            field.selectorBoard.tiles = []
-            field.selectorBoard.allSelectorTiles = []
-            self.removeWall()
-            field.centralScene.removeTiles()
-            field.game = None
-            field.refresh()
-
-    def removeWall(self):
-        """remote the wall"""
+    def removeGameFromPlayfield(self):
+        """remove the wall and player boards"""
+        for player in self.players:
+            if player.handBoard:
+                player.clearHand()
+                player.handBoard.hide()
         if self.wall:
             self.wall.hide()
             self.wall = None
@@ -199,8 +274,70 @@ class Game(object):
         """does this game instance belong to a player (as opposed to the game server)?"""
         return self.belongsToRobotPlayer() or self.belongsToHumanPlayer()
 
+    def assignPlayers(self, playerNames):
+        """the server tells us the seating order and player names"""
+        pairs = []
+        for idx, pair in enumerate(playerNames):
+            if isinstance(pair, basestring):
+                wind, name = WINDS[idx], pair
+            else:
+                wind, name = pair
+            pairs.append((wind, name))
+
+        field = InternalParameters.field
+        if not self.players:
+            if field:
+                self.players = field.genPlayers()
+            else:
+                self.players = Players([Player(self) for idx in range(4)])
+            for idx, pair in enumerate(pairs):
+                wind, name = pair
+                player = self.players[idx]
+                Players.createIfUnknown(name)
+                player.wind = wind
+                player.name = name
+        else:
+            for idx, pair in enumerate(playerNames):
+                wind, name = pair
+                self.players.byName(name).wind = wind
+        if self.client and self.client.username:
+            self.myself = self.players.byName(self.client.username)
+        self.sortPlayers()
+
+    def assignVoices(self):
+        """now we have all remote user voices"""
+        assert self.belongsToHumanPlayer()
+        available = Voice.availableVoices()[:]
+        # available is without transferred human voices
+        for player in self.players:
+            if player.voice and player.voice.oggFiles():
+                # remote human player sent her voice, or we are human and have a voice
+                if Debug.sound and player != self.myself:
+                    logDebug('%s got voice from opponent: %s' % (player.name, player.voice))
+            else:
+                player.voice = Voice.locate(player.name)
+                if player.voice:
+                    if Debug.sound:
+                        logDebug('%s has own local voice %s' % (player.name, player.voice))
+            if player.voice:
+                for voice in Voice.availableVoices():
+                    if voice in available and voice.md5sum == player.voice.md5sum:
+                        # if the local voice is also predefined,
+                        # make sure we do not use both
+                        available.remove(voice)
+        # for the other players use predefined voices in preferred language. Only if
+        # we do not have enough predefined voices, look again in locally defined voices
+        predefined = [x for x in available if x.language() != 'local']
+        predefined.extend(available)
+        for player in self.players:
+            if player.voice is None and predefined:
+                player.voice = predefined.pop(0)
+                if Debug.sound:
+                    logDebug('%s gets one of the still available voices %s' % (player.name, player.voice))
+
     def shufflePlayers(self):
         """assign random seats to the players and assign winds"""
+        self.players.sort(key=lambda x:x.name)
         self.randomGenerator.shuffle(self.players)
         for player, wind in zip(self.players, WINDS):
             player.wind = wind
@@ -234,8 +371,6 @@ class Game(object):
         if InternalParameters.field:
             fieldAttributes = list([(p.handBoard, p.front) for p in players])
         players.sort(key=Game.windOrder)
-        for idx, player in enumerate(players):
-            player.idx = idx
         if self.belongsToHumanPlayer():
             myName = self.myself.name
             while players[0].name != myName:
@@ -249,6 +384,7 @@ class Game(object):
             for idx, player in enumerate(players):
                 player.handBoard, player.front = fieldAttributes[idx]
                 player.handBoard.player = player
+        self.activePlayer = self.players['E']
 
     @staticmethod
     def _newGameId():
@@ -301,12 +437,21 @@ class Game(object):
             self.ruleset.rulesetId = self.ruleset.newId(used=True)
             self.ruleset.save()
 
+    def setHandSeed(self):
+        """set seed to a reproducable value, independent of what happend
+        in previous hands/rounds.
+        This makes it easier to reproduce game situations
+        in later hands without having to exactly replay all previous hands"""
+        if self.seed is not None:
+            seedFactor = (self.roundsFinished + 1) * 10000 + self.rotated * 1000 + self.notRotated * 100
+            self.randomGenerator.seed(self.seed * seedFactor)
+
     def prepareHand(self):
         """prepares the next hand"""
         del self.moves[:]
         if self.finished():
             if InternalParameters.field and isAlive(InternalParameters.field):
-                InternalParameters.field.refresh()
+                InternalParameters.field.updateGUI()
             self.close()
         else:
             for player in self.players:
@@ -315,14 +460,18 @@ class Game(object):
             if not self.isScoringGame():
                 self.sortPlayers()
             self.hidePopups()
-            self.activePlayer = self.players['E']
+            self.setHandSeed()
             self.wall.build()
-            HandContent.clearCache()
-            self.dangerousTiles = set()
-            self.discardedTiles.clear()
-            assert self.visibleTiles.count() == 0
+
+    def initHand(self):
+        """directly before starting"""
+        Hand.clearCache(self)
+        self.dangerousTiles = list()
+        self.discardedTiles.clear()
+        assert self.visibleTiles.count() == 0
         if InternalParameters.field:
             InternalParameters.field.prepareHand()
+        self.setHandSeed()
 
     def hidePopups(self):
         """hide all popup messages"""
@@ -334,6 +483,9 @@ class Game(object):
         self.__payHand()
         self.__saveScores()
         self.handctr += 1
+        self.notRotated += 1
+        self.roundHandCount += 1
+        self.handDiscardCount = 0
         if self.winner and self.winner.wind == 'E':
             self.eastMJCount += 1
 
@@ -354,18 +506,29 @@ class Game(object):
         with Transaction():
             for player in self.players:
                 if player.handContent:
-                    manualrules = '||'.join(x.name for x, meld in player.handContent.usedRules)
+                    manualrules = '||'.join(x.rule.name for x in player.handContent.usedRules)
                 else:
                     manualrules = m18n('Score computed manually')
                 Query("INSERT INTO SCORE "
                     "(game,hand,data,manualrules,player,scoretime,won,prevailing,wind,"
-                    "points,payments, balance,rotated) "
-                    "VALUES(%d,%d,?,?,%d,'%s',%d,'%s','%s',%d,%d,%d,%d)" % \
+                    "points,payments, balance,rotated,notrotated) "
+                    "VALUES(%d,%d,?,?,%d,'%s',%d,'%s','%s',%d,%d,%d,%d,%d)" % \
                     (self.gameid, self.handctr, player.nameid,
                         scoretime, int(player == self.winner),
                         WINDS[self.roundsFinished % 4], player.wind, player.handTotal,
-                        player.payment, player.balance, self.rotated),
+                        player.payment, player.balance, self.rotated, self.notRotated),
                     list([player.handContent.string, manualrules]))
+                if Debug.scores:
+                    self.debug('%s: roundwind=%s playerwind=%s handTotal=%s balance=%s' % (
+                        player, WINDS[self.roundsFinished % 4], player.wind,
+                        player.handTotal, player.balance))
+                for usedRule in player.handContent.usedRules:
+                    rule = usedRule.rule
+                    if rule.score.limits:
+                        tag = rule.function.__class__.__name__
+                        if hasattr(rule.function, 'limitHand'):
+                            tag = rule.function.limitHand.__class__.__name__
+                        self.addCsvTag(tag)
 
     def savePenalty(self, player, offense, amount):
         """save computed values to database, update score table and balance in status line"""
@@ -375,21 +538,18 @@ class Game(object):
         with Transaction():
             Query("INSERT INTO SCORE "
                 "(game,penalty,hand,data,manualrules,player,scoretime,"
-                "won,prevailing,wind,points,payments, balance,rotated) "
-                "VALUES(%d,1,%d,?,?,%d,'%s',%d,'%s','%s',%d,%d,%d,%d)" % \
+                "won,prevailing,wind,points,payments, balance,rotated,notrotated) "
+                "VALUES(%d,1,%d,?,?,%d,'%s',%d,'%s','%s',%d,%d,%d,%d,%d)" % \
                 (self.gameid, self.handctr, player.nameid,
                     scoretime, int(player == self.winner),
                     WINDS[self.roundsFinished % 4], player.wind, 0,
-                    amount, player.balance, self.rotated),
+                    amount, player.balance, self.rotated, self.notRotated),
                 list([player.handContent.string, offense.name]))
         if InternalParameters.field:
-            InternalParameters.field.refresh()
+            InternalParameters.field.updateGUI()
 
     def maybeRotateWinds(self):
         """if needed, rotate winds, exchange seats. If finished, update database"""
-        if self.belongsToPlayer():
-            # the server does that and tells us to rotate
-            return False
         if not self.winner:
             return False
         result = self.winner.wind != 'E' or self.eastMJCount == 9
@@ -400,11 +560,13 @@ class Game(object):
     def rotateWinds(self):
         """rotate winds, exchange seats. If finished, update database"""
         self.rotated += 1
+        self.notRotated = 0
         self.eastMJCount = 0
         if self.rotated == 4:
             if not self.finished():
                 self.roundsFinished += 1
             self.rotated = 0
+            self.roundHandCount = 0
         if self.finished():
             endtime = datetime.datetime.now().replace(microsecond=0).isoformat()
             with Transaction():
@@ -420,6 +582,19 @@ class Game(object):
                 # exchange seats between rounds
                 self.__exchangeSeats()
 
+    def debug(self, msg, btIndent=None):
+        """prepend game id"""
+        if self.belongsToRobotPlayer():
+            prefix = 'R'
+        elif self.belongsToHumanPlayer():
+            prefix = 'C'
+        elif self.belongsToGameServer():
+            prefix = 'S'
+        else:
+            logDebug(msg, btIndent=btIndent)
+            return
+        logDebug('%s%s: %s' % (prefix, self.handId(), msg), withGamePrefix=False, btIndent=btIndent)
+
     @staticmethod
     def __getNames(record):
         """get name ids from record
@@ -437,6 +612,7 @@ class Game(object):
     @staticmethod
     def loadFromDB(gameid, client=None, what=None, cacheRuleset=False):
         """load game by game id and return a new Game instance"""
+        InternalParameters.logPrefix = 'S' if InternalParameters.isServer else 'C'
         qGame = Query("select p0,p1,p2,p3,ruleset,seed from game where id = %d" % gameid)
         if not qGame.records:
             return None
@@ -448,7 +624,7 @@ class Game(object):
         Players.load() # we want to make sure we have the current definitions
         what = what or Game
         game = what(Game.__getNames(qGame.records[0]), ruleset, gameid=gameid,
-                client=client, seed=qGame.records[0][5])
+                client=client, wantedGame=qGame.records[0][5])
         qLastHand = Query("select hand,rotated from score where game=%d and hand="
             "(select max(hand) from score where game=%d)" % (gameid, gameid))
         if qLastHand.records:
@@ -490,27 +666,36 @@ class Game(object):
 
     def finished(self):
         """The game is over after minRounds completed rounds"""
-        return self.roundsFinished >= self.ruleset.minRounds
+        if self.ruleset:
+            # while initialising Game, ruleset might be None
+            return self.roundsFinished >= self.ruleset.minRounds
 
     def __payHand(self):
         """pay the scores"""
+        # pylint: disable=R0912
+        # too many branches
         winner = self.winner
         if winner:
             winner.wonCount += 1
-            if winner.usedDangerousFrom:
+            guilty = winner.usedDangerousFrom
+            if guilty:
+                payAction = self.ruleset.findAction('payforall')
+            if guilty and payAction:
                 if Debug.dangerousGame:
-                    logDebug('Seed %d,round %d, hand %d, winner %s:%s pays for all' % \
-                                (self.seed, self.roundsFinished+1, self.handctr, winner, winner.usedDangerousFrom))
+                    self.debug('%s: winner %s. %s pays for all' % \
+                                (self.handId(), winner, guilty))
+                guilty.handContent.usedRules.append((payAction, None))
                 score = winner.handTotal
-                if winner.wind == 'E':
-                    score = score * 6
-                else:
-                    score = score * 4
-                winner.usedDangerousFrom.getsPayment(-score)
+                score = score * 6 if winner.wind == 'E' else score * 4
+                guilty.getsPayment(-score)
                 winner.getsPayment(score)
                 return
 
         for player1 in self.players:
+            if Debug.explain:
+                self.debug('%s: %s' % (player1, player1.handContent.string))
+                for line in player1.handContent.explain():
+                    self.debug('   %s' % (line))
             for player2 in self.players:
                 if id(player1) != id(player2):
                     if player1.wind == 'E' or player2.wind == 'E':
@@ -537,6 +722,7 @@ class Game(object):
         """sets random living and kongBox
         sets divideAt: an index for the wall break"""
         if self.belongsToGameServer():
+            self.wall.tiles.sort(key=tileKey)
             self.randomGenerator.shuffle(self.wall.tiles)
         breakWall = self.randomGenerator.randrange(4)
         sideLength = len(self.wall.tiles) // 4
@@ -547,16 +733,31 @@ class Game(object):
             self.divideAt -= 1
         self.divideAt %= len(self.wall.tiles)
 
+    def dangerousFor(self, forPlayer, tile):
+        """returns a list of explaining texts if discarding tile
+        would be Dangerous game for forPlayer. One text for each
+        reason - there might be more than one"""
+        if isinstance(tile, Tile):
+            tile = tile.element
+        tile = tile.lower()
+        result = []
+        for dang, txt in self.dangerousTiles:
+            if tile in dang:
+                result.append(txt)
+        for player in forPlayer.others():
+            for dang, txt in player.dangerousTiles:
+                if tile in dang:
+                    result.append(txt)
+        return result
+
     def computeDangerous(self, playerChanged=None):
         """recompute gamewide dangerous tiles. Either for playerChanged or for all players"""
-        self.dangerousTiles = set()
+        self.dangerousTiles = list()
         if playerChanged:
             playerChanged.findDangerousTiles()
         else:
             for player in self.players:
                 player.findDangerousTiles()
-        for player in self.players:
-            self.dangerousTiles |= player.dangerousTiles
         self._endWallDangerous()
 
     def _endWallDangerous(self):
@@ -564,7 +765,10 @@ class Game(object):
         if len(self.wall.living) <=5:
             allTiles = [x for x in defaultdict.keys(elements.occurrence) if x[0] not in 'fy']
             # see http://www.logilab.org/ticket/23986
-            self.dangerousTiles |= set(x for x in allTiles if x not in self.visibleTiles)
+            invisibleTiles = set(x for x in allTiles if x not in self.visibleTiles)
+            msg = m18n('Short living wall: Tile is invisible, hence dangerous')
+            self.dangerousTiles = list(x for x in self.dangerousTiles if x[1] != msg)
+            self.dangerousTiles.append((invisibleTiles, msg))
 
     def appendMove(self, player, command, kwargs):
         """append a Move object to self.moves"""
@@ -574,8 +778,8 @@ class ScoringGame(Game):
     """we play manually on a real table with real tiles and use
     kajongg only for scoring"""
 
-    def __init__(self, names, ruleset, gameid=None, client=None, seed=None):
-        Game.__init__(self, names, ruleset, gameid=gameid, client=client, seed=seed)
+    def __init__(self, names, ruleset, gameid=None, client=None, wantedGame=None):
+        Game.__init__(self, names, ruleset, gameid=gameid, client=client, wantedGame=wantedGame)
         field = InternalParameters.field
         field.selectorBoard.load(self)
         self.prepareHand()
@@ -609,20 +813,27 @@ class PlayingGame(Game):
 class RemoteGame(PlayingGame):
     """this game is played using the computer"""
     # pylint: disable=R0913
-    # pylint too many arguments
-    def __init__(self, names, ruleset, gameid=None, seed=None, shouldSave=True, \
+    # pylint: disable=R0904
+    # pylint too many arguments, too many public methods
+    def __init__(self, names, ruleset, gameid=None, wantedGame=None, shouldSave=True, \
             client=None, playOpen=False, autoPlay=False):
         """a new game instance, comes from database if gameid is set"""
         self.__activePlayer = None
         self.prevActivePlayer = None
         self.defaultNameBrush = None
         PlayingGame.__init__(self, names, ruleset, gameid,
-            seed=seed, shouldSave=shouldSave, client=client)
+            wantedGame=wantedGame, shouldSave=shouldSave, client=client)
         self.playOpen = playOpen
         self.autoPlay = autoPlay
-        for player in self.players:
-            if player.name.startswith('ROBOT'):
-                player.voice = Voice(player.name)
+        myself = self.myself
+        if self.belongsToHumanPlayer() and myself:
+            myself.voice = Voice.locate(myself.name)
+            if myself.voice:
+                if Debug.sound:
+                    logDebug('RemoteGame: myself %s gets voice %s' % (myself.name, myself.voice))
+            else:
+                if Debug.sound:
+                    logDebug('myself %s gets no voice'% (myself.name))
 
     @staticmethod
     def loadFromDB(gameid, client=None, what=None, cacheRuleset=False):
@@ -639,6 +850,8 @@ class RemoteGame(PlayingGame):
             # pylint: disable=W0212
             if self.__activePlayer != player:
                 self.prevActivePlayer = self.__activePlayer
+                if self.prevActivePlayer:
+                    self.prevActivePlayer.hidePopup()
                 self.__activePlayer = player
                 if InternalParameters.field: # mark the name of the active player in blue
                     for player in self.players:
@@ -668,47 +881,37 @@ class RemoteGame(PlayingGame):
             while len(player.concealedTileNames) != 13:
                 player.addConcealedTiles(self.wall.deal())
 
-    def pickedTile(self, player, deadEnd, tileName=None):
-        """got a tile from wall"""
-        self.activePlayer = player
-        tile = self.wall.deal([tileName], deadEnd=deadEnd)[0]
-        player.addConcealedTiles(tile)
-        player.lastTile = tile.element
-        if deadEnd:
-            player.lastSource = 'e'
+    def __concealedTileName(self, tileName):
+        """tileName has been discarded, by which name did we know it?"""
+        player = self.activePlayer
+        if self.myself and player != self.myself and not self.playOpen:
+            # we are human and server tells us another player discarded a tile. In our
+            # game instance, tiles in handBoards of other players are unknown
+            player.concealedTileNames[0] = tileName
+            result = 'Xy'
         else:
-            self.lastDiscard = None
-            player.lastSource = 'w'
-        return tile
-
-    def showField(self):
-        """show remote game in field"""
-        self.wall.divide()
-        if InternalParameters.field:
-            InternalParameters.field.setWindowTitle(m18n('Game <numid>%1</numid>', str(self.seed)) + ' - Kajongg')
-            InternalParameters.field.discardBoard.setRandomPlaces(self.randomGenerator)
-            for tableList in InternalParameters.field.tableLists:
-                tableList.hide()
-            InternalParameters.field.tableLists = []
+            result = tileName
+        if not tileName in player.concealedTileNames:
+            raise Exception('I am %s. Player %s is told to show discard of tile %s but does not have it, he has %s' % \
+                           (self.myself.name if self.myself else 'None',
+                            player.name, result, player.concealedTileNames))
+        return result
 
     def hasDiscarded(self, player, tileName):
         """discards a tile from a player board"""
+        # pylint: disable=R0912
+        # too many branches
         if player != self.activePlayer:
             raise Exception('Player %s discards but %s is active' % (player, self.activePlayer))
         self.discardedTiles[tileName.lower()] += 1
         player.discarded.append(tileName)
-        if self.myself and player != self.myself and not self.playOpen:
-            # we are human and server tells us another player discarded a tile. In our
-            # game instance, tiles in handBoards of other players are unknown
-            concealedTileName = 'Xy'
-            player.concealedTileNames[0] = tileName
-        else:
-            concealedTileName = tileName
+        concealedTileName = self.__concealedTileName(tileName) # has side effect, needs to be called
         if InternalParameters.field:
             if player.handBoard.focusTile and player.handBoard.focusTile.element == tileName:
                 self.lastDiscard = player.handBoard.focusTile
             else:
-                matchingTiles = sorted(player.handBoard.tilesByElement(concealedTileName), key=lambda x:x.xoffset)
+                matchingTiles = sorted(player.handBoard.tilesByElement(concealedTileName),
+                    key=lambda x:x.xoffset)
                 # if an opponent player discards, we want to discard from the right end of the hand
                 # thus minimizing tile movement
                 self.lastDiscard = matchingTiles[-1]
@@ -716,40 +919,33 @@ class RemoteGame(PlayingGame):
             InternalParameters.field.discardBoard.discardTile(self.lastDiscard)
         else:
             self.lastDiscard = Tile(tileName)
-        if not tileName in player.concealedTileNames:
-            raise Exception('I am %s. Player %s is told to show discard of tile %s but does not have it, he has %s' % \
-                           (self.myself.name if self.myself else 'None',
-                            player.name, concealedTileName, player.concealedTileNames))
         player.remove(tile=self.lastDiscard)
-        if tileName.lower() in self.dangerousTiles:
+        if any(tileName.lower() in x[0] for x in self.dangerousTiles):
             self.computeDangerous()
         else:
             self._endWallDangerous()
+        self.handDiscardCount += 1
         if InternalParameters.field:
             for tile in player.handBoard.tiles:
                 tile.focusable = False
-            if player == self.myself:
-                player.hidePopup()
+
+    def checkTarget(self):
+        """check if we reached the point defined by --game.
+        If we did, disable autoPlay"""
+        parts = self.wantedGame.split('/')
+        if len(parts) > 1:
+            discardCount = int(parts[2]) if len(parts) > 2 else 0
+            if self.handId().split('/')[-1] == parts[1] \
+               and self.handDiscardCount >= int(discardCount):
+                self.autoPlay = False
+                self.wantedGame = parts[0] # --game has been processed
+                if InternalParameters.field: # mark the name of the active player in blue
+                    InternalParameters.field.actionAutoPlay.setChecked(False)
 
     def saveHand(self):
         """server told us to save this hand"""
         for player in self.players:
-            player.handContent = player.computeHandContent()
+            player.handContent = player.computeHand()
             if player == self.winner:
-                assert player.handContent.maybeMahjongg()
+                assert player.handContent.won
         Game.saveHand(self)
-
-    def close(self, callback=None):
-        """log off from the server"""
-        InternalParameters.autoPlay = False # do that only for the first game
-        if self.client:
-            deferred = self.client.logout() if self.client.perspective else None
-            self.client = None
-            if deferred:
-                deferred.addBoth(self.hide)
-                if callback:
-                    deferred.addBoth(callback)
-                return
-        self.hide()
-        if callback:
-            callback()
