@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
- (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+ (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,34 +24,45 @@ from PyQt4.QtGui import QMenu, QCursor
 from PyQt4.QtGui import QGraphicsSimpleTextItem
 from tile import Tile, swapTitle
 from meld import Meld, EXPOSED, CONCEALED, REST, tileKey, elementKey, shortcuttedMeldName
-from scoringengine import HandContent
+from hand import Hand
 from board import Board, rotateCenter
 
-from util import m18n
+from util import m18n, logDebug, isAlive
 import common
-from common import InternalParameters
+from common import InternalParameters, Debug
 from animation import animate
 
 class TileAttr(object):
     """a helper class for syncing the hand board, holding relevant tile attributes"""
-    def __init__(self, element, xoffset=None, yoffset=None):
-        if isinstance(element, Tile):
-            self.element = element.element
-            self.xoffset = element.xoffset
-            self.yoffset = element.yoffset
-            self.dark = element.dark
-            self.focusable = element.focusable
+    def __init__(self, hand, meld=None, idx=None, xoffset=None, yoffset=None):
+        if isinstance(hand, Tile):
+            self.element = hand.element
+            self.xoffset = hand.xoffset
+            self.yoffset = hand.yoffset
+            self.dark = hand.dark
+            self.focusable = hand.focusable
         else:
-            self.element = element
+            self.element = meld.pairs[idx] if idx is not None else meld
             self.xoffset = xoffset
             self.yoffset = yoffset
-            self.dark = False
-            self.focusable = True
-            isScoringGame = InternalParameters.field.game.isScoringGame()
+            player = hand.player
+            isScoringGame = player.game.isScoringGame()
             if yoffset == 0:
-                self.dark = element.istitle()
+                self.dark = self.element.istitle()
             else:
-                self.dark = element == 'Xy' or isScoringGame
+                self.dark = self.element == 'Xy' or isScoringGame
+            self.focusable = True
+            if isScoringGame:
+                self.focusable = idx == 0
+            else:
+                self.focusable = (self.element[0] not in 'fy'
+                    and self.element != 'Xy'
+                    and player == player.game.activePlayer
+                    and player == player.game.myself
+                    and (meld.state == CONCEALED
+                    and (len(meld) < 4 or meld.meldType == REST)))
+            if self.element in Debug.focusable:
+                logDebug('TileAttr %s:%s' % (self.element, self.focusable))
 
     def __str__(self):
         return '%s %.1f/%.1f%s%s' % (self.element, self.xoffset, self.yoffset, ' dark' if self.dark else '', \
@@ -64,11 +75,11 @@ class HandBoard(Board):
     # pylint: disable=R0902
     # pylint - we need more than 10 instance attributes
     def __init__(self, player):
-        self.exposedMeldDistance = 0.2
+        self.exposedMeldDistance = 0.15
         self.concealedMeldDistance = 0.0
         self.lowerY = 1.0
         self.player = player
-        Board.__init__(self, 15.4, 2.0, InternalParameters.field.tileset)
+        Board.__init__(self, 15.6, 2.0, InternalParameters.field.tileset)
         self.isHandBoard = True
         self.tileDragEnabled = False
         self.setParentItem(player.front)
@@ -109,7 +120,7 @@ class HandBoard(Board):
                     self.lowerY = 1.2
                 else:
                     self.lowerY = 1.0
-                self.setRect(15.4, 1.0 + self.lowerY)
+                self.setRect(15.6, 1.0 + self.lowerY)
                 self._reload(self.tileset, showShadows=value)
                 self.sync()
         return property(**locals())
@@ -128,9 +139,12 @@ class HandBoard(Board):
 
     def setEnabled(self, enabled):
         """enable/disable this board"""
-        self.tileDragEnabled = enabled and \
-        (self.player.game.isScoringGame() or self.player == self.player.game.myself)
-        QGraphicsRectItem.setEnabled(self, enabled)
+        if isAlive(self):
+            # aborting a running game: the underlying C++ object might
+            # already have been destroyed
+            self.tileDragEnabled = enabled and \
+            (self.player.game.isScoringGame() or self.player == self.player.game.myself)
+            QGraphicsRectItem.setEnabled(self, enabled)
 
     def showMoveHelper(self, visible=True):
         """show help text In empty HandBoards"""
@@ -183,7 +197,7 @@ class HandBoard(Board):
         for meld in self.player.concealedMelds + self.player.exposedMelds:
             if tile in meld.tiles:
                 return meld
-        assert False, 'meldWithThile: %s' % str(tile)
+        assert False, 'meldWithTile: %s' % str(tile)
 
     def dragObject(self, tile):
         """if user wants to drag tile, he really might want to drag the meld"""
@@ -282,7 +296,7 @@ class HandBoard(Board):
 
     def lowerHalfTiles(self):
         """returns a list with all single tiles of the lower half melds without boni"""
-        return list(x for x in self.tiles if x.yoffset > 0)
+        return list(x for x in self.tiles if x.yoffset > 0 and not x.isBonus())
 
     def newTilePositions(self):
         """returns list(TileAttr) for all tiles except bonus tiles.
@@ -296,9 +310,10 @@ class HandBoard(Board):
             if self.player.concealedMelds:
                 newLowerMelds = sorted(self.player.concealedMelds)
             else:
-                tileStr = ''.join(self.player.concealedTileNames)
-                content = HandContent.cached(self.player.game.ruleset, tileStr)
-                newLowerMelds = list(Meld(x) for x in content.sortedMelds.split())
+                tileStr = 'R' + ''.join(self.player.concealedTileNames)
+                handStr = ' '.join([tileStr, self.player.mjString()])
+                content = Hand.cached(self.player, handStr)
+                newLowerMelds = list(Meld(x) for x in content.sortedMeldsContent.split())
                 if newLowerMelds:
                     if self.rearrangeMelds:
                         if newLowerMelds[0].pairs[0] == 'Xy':
@@ -310,18 +325,8 @@ class HandBoard(Board):
             meldDistance = self.concealedMeldDistance if yPos else self.exposedMeldDistance
             meldX = 0
             for meld in melds:
-                for idx, tileName in enumerate(meld.pairs):
-                    newTile = TileAttr(tileName, meldX, yPos)
-                    if isScoringGame:
-                        newTile.focusable = idx == 0
-                    else:
-                        newTile.focusable = (tileName[0] not in 'fy'
-                            and tileName != 'Xy'
-                            and self.player == self.player.game.activePlayer
-                            and self.player == self.player.game.myself
-                            and (meld.state == CONCEALED
-                            and (len(meld) < 4 or meld.meldType == REST)))
-                    result.append(newTile)
+                for idx in range(len(meld)):
+                    result.append(TileAttr(self, meld, idx, meldX, yPos))
                     meldX += 1
                 meldX += meldDistance
         return sorted(result, key=lambda x: x.yoffset * 100 + x.xoffset)
@@ -347,7 +352,6 @@ class HandBoard(Board):
         result = list()
         for bonus in sorted(newBonusTiles, key=tileKey):
             bonus.xoffset, bonus.yoffset = xPos, bonusY
-            bonus.focusable = self.player.game.isScoringGame()
             bonus.dark = False
             result.append(bonus)
             xPos += 1
@@ -393,20 +397,40 @@ class HandBoard(Board):
             self.__avoidCrossingMovements(result)
         return result
 
+    def __movingPlaces(self, places):
+        """filter out the left parts of the rows which do not change
+        at all"""
+        rows = [[], []]
+        for idx, yOld in enumerate([0, self.lowerY]):
+            rowPlaces = [x for x in places.items() if x[0].yoffset == yOld]
+            rowPlaces = sorted(rowPlaces, key=lambda x: x[0].xoffset)
+            smallestX = 999
+            for tile, newPos in places.items():
+                if tile.xoffset != newPos.xoffset or tile.yoffset != newPos.yoffset:
+                    if newPos.yoffset == yOld:
+                        smallestX = min(smallestX, newPos.xoffset)
+                    else:
+                        smallestX = min(smallestX, tile.xoffset)
+            rows[idx] = [x for x in rowPlaces if x[0].xoffset >= smallestX and x[1].xoffset >= smallestX]
+        result = dict(rows[0])
+        result.update(dict(rows[1]))
+        return result
+
     def __avoidCrossingMovements(self, places):
         """"the above is a good approximation but if the board already had more
         than one identical tile they often switch places - this should not happen.
         So for each element, we make sure that the left-right order is still the
         same as before. For this check, ignore all new tiles"""
+        movingPlaces = self.__movingPlaces(places)
         for yOld in 0, self.lowerY:
             for yNew in 0, self.lowerY:
-                items = [x for x in places.items() \
+                items = [x for x in movingPlaces.items() \
                          if (x[0].board == self) \
                             and x[0].yoffset == yOld \
                             and x[1] and x[1].yoffset == yNew \
                             and not x[0].isBonus()]
                 for element in set(x[1].element for x in items):
-                    items = [x for x in places.items() if x[1].element == element]
+                    items = [x for x in movingPlaces.items() if x[1].element == element]
                     if len(items) > 1:
                         oldList = sorted(list(x[0] for x in items), key=lambda x:bool(x.board!=self)*1000+x.xoffset)
                         newList = sorted(list(x[1] for x in items), key=lambda x:x.xoffset)
