@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@ from PyKDE4.kdecore import KUser, KGlobal, KStandardDirs, \
     KCmdLineOptions
 
 
-from PyKDE4.kdecore import i18n, i18nc, i18np, ki18n
+from PyKDE4.kdecore import i18n, i18nc, ki18n
 from PyKDE4.kdeui import KMessageBox, KIcon, KLineEdit, \
     KConfigSkeleton, KDialogButtonBox, KAction, KStandardAction, \
     KApplication, KToggleFullScreenAction, KXmlGuiWindow, \
