@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,12 +21,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 import sys
 from collections import defaultdict
 
-from util import logException, logWarning, logDebug
+from util import logException, logWarning, m18n, m18nc
 from common import WINDS, InternalParameters, elements, IntDict, Debug
 from query import Transaction, Query
-from tile import Tile, offsetTiles
-from meld import Meld, CONCEALED, PUNG
-from scoringengine import HandContent
+from tile import Tile
+from meld import Meld, CONCEALED, PUNG, hasChows, meldsContent
+from hand import Hand
 
 class Players(list):
     """a list of players where the player can also be indexed by wind.
@@ -65,7 +65,7 @@ class Players(list):
         for player in self:
             if player.name == playerName:
                 return player
-        logException("no player has name %s" % playerName)
+        logException("no player has name %s - we have %s" % (playerName, [x.name for x in self]))
 
     @staticmethod
     def load():
@@ -124,8 +124,7 @@ class Player(object):
 
     def speak(self, text):
         """speak if we have a voice"""
-        if self.voice:
-            self.voice.speak(text)
+        pass
 
     def clearHand(self):
         """clear player attributes concerning the current hand"""
@@ -136,16 +135,18 @@ class Player(object):
         self.discarded = []
         self.visibleTiles.clear()
         self.handContent = None
-        self.lastTile = 'xx'
+        self.originalCallingHand = None
+        self.lastTile = None
         self.lastSource = '1'
         self.lastMeld = Meld()
         self.mayWin = True
         self.__payment = 0
         self.originalCall = False
-        self.dangerousTiles = set() # all elements should be lowercase
+        self.dangerousTiles = list()
         self.claimedNoChoice = False
         self.playedDangerous = False
         self.usedDangerousFrom = None
+        self.isCalling = False
 
     @apply
     def lastSource(): # pylint: disable=E0202
@@ -177,7 +178,7 @@ class Player(object):
 
     @apply
     def handTotal():
-        """the name id of this player"""
+        """the hand total of this player"""
         def fget(self):
             if self.hasManualScore():
                 spValue = InternalParameters.field.scoringDialog.spValues[self.idx]
@@ -235,6 +236,19 @@ class Player(object):
     def __unicode__(self):
         return u'{name:<10} {wind}'.format(name=self.name[:10], wind=self.wind)
 
+    def pickedTile(self, deadEnd, tileName=None):
+        """got a tile from wall"""
+        self.game.activePlayer = self
+        tile = self.game.wall.deal([tileName], deadEnd=deadEnd)[0]
+        self.addConcealedTiles(tile)
+        self.lastTile = tile.element
+        if deadEnd:
+            self.lastSource = 'e'
+        else:
+            self.game.lastDiscard = None
+            self.lastSource = 'w'
+        return tile
+
     def addConcealedTiles(self, data):
         """add to my tiles and sync the hand board"""
         assert isinstance(data, (Tile, list)), data
@@ -290,13 +304,15 @@ class Player(object):
                 if id(myTile) == id(meld):
                     melds.pop(idx)
 
-    def hasConcealedTiles(self, tileNames):
+    def hasConcealedTiles(self, tileNames, within=None):
         """do I have those concealed tiles?"""
-        concealedTileNames = self.concealedTileNames[:]
+        if within is None:
+            within = self.concealedTileNames
+        within = within[:]
         for tileName in tileNames:
-            if tileName not in concealedTileNames:
+            if tileName not in within:
                 return False
-            concealedTileNames.remove(tileName)
+            within.remove(tileName)
         return True
 
     def showConcealedTiles(self, tileNames, show=True):
@@ -352,35 +368,45 @@ class Player(object):
             return True
         if 'Xy' in self.concealedTileNames:
             return True
-        self.handContent = self.computeHandContent()
+        self.handContent = self.computeHand()
         if str(self.handContent) == score:
             return True
-        logDebug('%s localScore:%s' % (self, self.handContent))
-        logDebug('%s serverScore:%s' % (self, score))
+        self.game.debug('%s localScore:%s' % (self, self.handContent))
+        self.game.debug('%s serverScore:%s' % (self, score))
         logWarning('Game %s: client and server disagree about scoring, see logfile for details' % self.game.seed)
         return False
 
     def mustPlayDangerous(self, exposing=None):
-        """returns True if the player has no choice. Exposing may be a meld
-        which will be exposed before we might play dangerous"""
+        """returns True if the player has no choice, otherwise False.
+        Exposing may be a meld which will be exposed before we might
+        play dangerous"""
+        if self == self.game.activePlayer and exposing and len(exposing) == 4:
+            # declaring a kong is never dangerous because we get
+            # an unknown replacement
+            return False
         afterExposed = list(x.lower() for x in self.concealedTileNames)
         if exposing:
             exposing = exposing[:]
-            exposing.remove(self.game.lastDiscard.element)
+            if self.game.lastDiscard:
+                # if this is about claiming a discarded tile, ignore it
+                # the player who discarded it is responsible
+                exposing.remove(self.game.lastDiscard.element)
             for tileName in exposing:
-                afterExposed.remove(tileName.lower())
-        return set(afterExposed) <= self.game.dangerousTiles
+                if tileName.lower() in afterExposed:
+                    # the "if" is needed for claimed pung
+                    afterExposed.remove(tileName.lower())
+        return all(self.game.dangerousFor(self, x) for x in afterExposed)
 
-    def exposeMeld(self, meldTiles, called=None):
+    def exposeMeld(self, meldTiles, calledTile=None):
         """exposes a meld with meldTiles: removes them from concealedTileNames,
         adds the meld to exposedMelds and returns it
-        called: we got the last tile for the meld from discarded, otherwise
+        calledTile: we got the last tile for the meld from discarded, otherwise
         from the wall"""
         game = self.game
         game.activePlayer = self
         allMeldTiles = meldTiles[:]
-        if called:
-            allMeldTiles.append(called.element if isinstance(called, Tile) else called)
+        if calledTile:
+            allMeldTiles.append(calledTile.element if isinstance(calledTile, Tile) else calledTile)
         if len(allMeldTiles) == 4 and allMeldTiles[0].islower():
             tile0 = allMeldTiles[0].lower()
             # we are adding a 4th tile to an exposed pung
@@ -395,44 +421,50 @@ class Player(object):
                 self.concealedTileNames.remove(meldTile)
             for meldTile in allMeldTiles:
                 self.visibleTiles[meldTile.lower()] += 1
-            meld.expose(bool(called))
+            meld.expose(bool(calledTile))
         self.exposedMelds.append(meld)
         game.computeDangerous(self)
-        adding = [called] if called else None
+        adding = [calledTile] if calledTile else None
         self.syncHandBoard(adding=adding)
         return meld
 
     def findDangerousTiles(self):
         """update the list of dangerous tile"""
-        dangerousTiles = set()
+        pName = m18nc('kajongg', self.name)
+        dangerous = list()
         expMeldCount = len(self.exposedMelds)
+        if expMeldCount >= 3:
+            if all(x in elements.greenHandTiles for x in self.visibleTiles):
+                dangerous.append((elements.greenHandTiles,
+                     m18n('Player %1 has 3 or 4 exposed melds, all are green', pName)))
+            color = defaultdict.keys(self.visibleTiles)[0][0]
+            # see http://www.logilab.org/ticket/23986
+            assert color.islower(), self.visibleTiles
+            if color in 'sbc':
+                if all(x[0] == color for x in self.visibleTiles):
+                    suitTiles = set([color+x for x in '123456789'])
+                    if self.visibleTiles.count(suitTiles) >= 9:
+                        dangerous.append((suitTiles, m18n('Player %1 may try a True Color Game', pName)))
+                elif all(x[1] in '19' for x in self.visibleTiles):
+                    dangerous.append((elements.terminals,
+                        m18n('Player %1 may try an All Terminals Game', pName)))
         if expMeldCount >= 2:
-            if expMeldCount >= 3:
-                if all(x in elements.greenHandTiles for x in self.visibleTiles):
-                    dangerousTiles |= elements.greenHandTiles
-                color = defaultdict.keys(self.visibleTiles)[0][0]
-                # see http://www.logilab.org/ticket/23986
-                assert color.islower(), self.visibleTiles
-                if color in 'sbc':
-                    if all(x[0] == color for x in self.visibleTiles):
-                        suitTiles = set([color+x for x in '123456789'])
-                        if self.visibleTiles.count(suitTiles) >= 9:
-                            dangerousTiles |= suitTiles
-                    elif all(x[1] in '19' for x in self.visibleTiles):
-                        dangerousTiles |= elements.terminals
-            elif expMeldCount >= 2:
-                windMelds = sum(self.visibleTiles[x] >=3 for x in elements.winds)
-                dragonMelds = sum(self.visibleTiles[x] >=3 for x in elements.dragons)
-                windsDangerous = dragonsDangerous = False
-                if windMelds + dragonMelds == expMeldCount and expMeldCount >= 3:
-                    windsDangerous = dragonsDangerous = True
-                windsDangerous = windsDangerous or windMelds == 3
-                dragonsDangerous = dragonsDangerous or dragonMelds == 2
-                if windsDangerous:
-                    dangerousTiles |= set(x for x in elements.winds if x not in self.visibleTiles)
-                if dragonsDangerous:
-                    dangerousTiles |= set(x for x in elements.dragons if x not in self.visibleTiles)
-        self.dangerousTiles = dangerousTiles
+            windMelds = sum(self.visibleTiles[x] >=3 for x in elements.winds)
+            dragonMelds = sum(self.visibleTiles[x] >=3 for x in elements.dragons)
+            windsDangerous = dragonsDangerous = False
+            if windMelds + dragonMelds == expMeldCount and expMeldCount >= 3:
+                windsDangerous = dragonsDangerous = True
+            windsDangerous = windsDangerous or windMelds >= 3
+            dragonsDangerous = dragonsDangerous or dragonMelds >= 2
+            if windsDangerous:
+                dangerous.append((set(x for x in elements.winds if x not in self.visibleTiles),
+                     m18n('Player %1 exposed many winds', pName)))
+            if dragonsDangerous:
+                dangerous.append((set(x for x in elements.dragons if x not in self.visibleTiles),
+                     m18n('Player %1 exposed many dragons', pName)))
+        self.dangerousTiles = dangerous
+        if dangerous and Debug.dangerousGame:
+            self.game.debug('dangerous:%s' % dangerous)
 
     def popupMsg(self, msg):
         """virtual: show popup on display"""
@@ -450,11 +482,11 @@ class Player(object):
         """virtual: colorize Name on wall"""
         pass
 
-    def setFocus(self, dummyTileName):
-        """virtual: sets focus on a tile"""
+    def getsFocus(self, dummyResults=None):
+        """virtual: player gets focus on his hand"""
         pass
 
-    def __mjString(self):
+    def mjString(self):
         """compile hand info into a string as needed by the scoring engine"""
         game = self.game
         assert game
@@ -471,62 +503,50 @@ class Player(object):
             wonChar = 'x'
         return ''.join([wonChar, winds, lastSource, declaration])
 
-    def __lastString(self):
-        """compile hand info into a string as needed by the scoring engine"""
-        game = self.game
-        if game is None:
-            return ''
-        if self != game.winner:
-            return ''
-        return 'L%s%s' % (self.lastTile, self.lastMeld.joined)
-
-    def computeHandContent(self, withTile=None, robbedTile=None, dummy=None):
-        """returns HandContent for this player"""
+    def computeHand(self, withTile=None, robbedTile=None, dummy=None):
+        """returns Hand for this player"""
         assert not (self.concealedMelds and self.concealedTileNames)
         assert not isinstance(self.lastTile, Tile)
-        prevLastTile = self.lastTile
+        assert not isinstance(withTile, Tile)
+        melds = ['R' + ''.join(self.concealedTileNames)]
         if withTile:
-            assert not isinstance(withTile, Tile)
-            self.lastTile = withTile
-        try:
-            melds = [''.join(self.concealedTileNames)]
-            if withTile:
-                melds[0] += withTile
-            melds.extend(x.joined for x in self.exposedMelds)
-            melds.extend(x.joined for x in self.concealedMelds)
-            melds.extend(''.join(x.element) for x in self.bonusTiles)
-            melds.append(self.__mjString())
-            melds.append(self.__lastString())
-        finally:
-            self.lastTile = prevLastTile
+            melds[0] += withTile
+        melds.extend(x.joined for x in self.exposedMelds)
+        melds.extend(x.joined for x in self.concealedMelds)
+        melds.extend(''.join(x.element) for x in self.bonusTiles)
+        mjString = self.mjString()
+        melds.append(mjString)
+        if mjString.startswith('M'):
+            melds.append('L%s%s' % (withTile or self.lastTile, self.lastMeld.joined))
         if self.game.eastMJCount == 8 and self == self.game.winner and self.wind == 'E':
             # eastMJCount will only be inced later, in saveHand
             rules = [self.game.ruleset.findRule('XEAST9X')]
         else:
             rules = None
-        return HandContent.cached(self.game.ruleset, ' '.join(melds), computedRules=rules, robbedTile=robbedTile)
+        return Hand.cached(self, ' '.join(melds), computedRules=rules, robbedTile=robbedTile)
 
-    def possibleChows(self):
-        """returns a unique list of lists with possible chow combinations"""
-        discard = self.game.lastDiscard.element
-        try:
-            value = int(discard[1])
-        except ValueError:
+    def possibleChows(self, tileName=None, within=None):
+        """returns a unique list of lists with possible claimable chow combinations"""
+        exposedChows = [x for x in self.exposedMelds if x.isChow()]
+        if len(exposedChows) >= self.game.ruleset.maxChows:
             return []
-        chows = []
-        for offsets in [(1, 2), (-2, -1), (-1, 1)]:
-            if value + offsets[0] >= 1 and value + offsets[1] <= 9:
-                chow = offsetTiles(discard, offsets)
-                if self.hasConcealedTiles(chow):
-                    chow.append(discard)
-                    if chow not in chows:
-                        chows.append(sorted(chow))
-        return chows
+        if tileName is None:
+            tileName = self.game.lastDiscard.element
+        if within is None:
+            within = self.concealedTileNames
+        within = within[:]
+        within.append(tileName)
+        return hasChows(tileName, within)
 
-    def possibleKongs(self, mayPlayDangerous=False):
+    def exposedChows(self):
+        """returns a list of exposed chows"""
+        return [x for x in self.exposedMelds if x.isChow()]
+
+    def possibleKongs(self):
         """returns a unique list of lists with possible kong combinations"""
         kongs = []
         if self == self.game.activePlayer:
+            # declaring a kong
             for tileName in set([x for x in self.concealedTileNames if x[0] not in 'fy']):
                 if self.concealedTileNames.count(tileName) == 4:
                     kongs.append([tileName] * 4)
@@ -534,13 +554,10 @@ class Player(object):
                         tileName.lower() * 3 in list(x.joined for x in self.exposedMelds):
                     kongs.append([tileName.lower()] * 3 + [tileName])
         if self.game.lastDiscard:
-            discard = self.game.lastDiscard.element
-            if self.concealedTileNames.count(discard.capitalize()) == 3:
-                must = self.mustPlayDangerous([discard] * 4)
-                if mayPlayDangerous or not must:
-                    kongs.append([discard.capitalize()] * 4)
-                elif Debug.dangerousGame:
-                    logDebug('%s: claiming Kong of %s would result in dangerous game' % (self, discard))
+            # claiming a kong
+            discardName = self.game.lastDiscard.element.capitalize()
+            if self.concealedTileNames.count(discardName) == 3:
+                kongs.append([discardName] * 4)
         return kongs
 
     def declaredMahJongg(self, concealed, withDiscard, lastTile, lastMeld):
@@ -548,20 +565,27 @@ class Player(object):
         assert not isinstance(lastTile, Tile)
         lastMeld = Meld(lastMeld) # do not change the original!
         self.game.winner = self
-        melds = [Meld(x) for x in concealed.split()]
         if withDiscard:
+            self.lastTile = withDiscard
+            self.lastMeld = lastMeld
+            assert withDiscard == self.game.lastDiscard.element, 'withDiscard: %s lastDiscard: %s' % (
+                withDiscard, self.game.lastDiscard.element)
             self.addConcealedTiles(self.game.lastDiscard)
-            self.lastTile = withDiscard.lower()
+            melds = [Meld(x) for x in concealed.split()]
             if self.lastSource != 'k':   # robbed the kong
                 self.lastSource = 'd'
             # the last claimed meld is exposed
+            assert lastMeld in melds, '%s: concealed=%s melds=%s lastMeld=%s lastTile=%s withDiscard=%s' % (
+                    self.concealedTileNames, concealed,
+                    meldsContent(melds), ''.join(lastMeld.pairs), lastTile, withDiscard)
             melds.remove(lastMeld)
+            self.lastTile = self.lastTile.lower()
             lastMeld.pairs.toLower()
             self.exposedMelds.append(lastMeld)
             for tileName in lastMeld.pairs:
                 self.visibleTiles[tileName] += 1
-            self.lastMeld = lastMeld
         else:
+            melds = [Meld(x) for x in concealed.split()]
             self.lastTile = lastTile
             self.lastMeld = lastMeld
         self.concealedMelds = melds
@@ -577,3 +601,34 @@ class Player(object):
             parts.extend([x.joined for x in self.exposedMelds])
         parts.extend(''.join(x.element) for x in self.bonusTiles)
         return ' '.join(parts)
+
+    def others(self):
+        """a list of the other 3 players"""
+        return (x for x in self.game.players if x != self)
+
+    def tileAvailable(self, tileName, hand):
+        """a count of how often tileName might still appear in the game
+        supposing we have hand"""
+        visible = self.game.discardedTiles.count([tileName.lower()])
+        for player in self.others():
+            visible += player.visibleTiles.count([tileName.capitalize()])
+            visible += player.visibleTiles.count([tileName.lower()])
+        for pair in hand.tileNames:
+            if pair.lower() == tileName.lower():
+                visible += 1
+        return 4 - visible
+
+    def violatesOriginalCall(self, tileName=None):
+        """called if discarding tileName (default=just discarded tile)
+        violates the Original Call"""
+        if not self.originalCall or not self.mayWin:
+            return False
+        if tileName is None:
+            if len(self.discarded) < 2:
+                return False
+            tileName = self.discarded[-1]
+        if self.lastTile.lower() != tileName.lower():
+            if Debug.originalCall:
+                self.game.debug('%s would violate OC with %s, lastTile=%s' % (self, tileName, self.lastTile))
+            return True
+        return False
