@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@ from util import logError, m18n, m18nc, isAlive, logWarning
 import common
 from common import WINDS, LIGHTSOURCES, InternalParameters
 import cgitb, tempfile, webbrowser
+from twisted.internet.defer import succeed
 
 class MyHook(cgitb.Hook):
     """override the standard cgitb hook: invoke the browser"""
@@ -41,11 +42,11 @@ class MyHook(cgitb.Hook):
 NOTFOUND = []
 
 try:
-    from PyQt4.QtCore import Qt, QVariant, SIGNAL, \
+    from PyQt4.QtCore import Qt, QVariant, \
         QEvent, QMetaObject, PYQT_VERSION_STR, QString
     from PyQt4.QtGui import QPushButton, QMessageBox
     from PyQt4.QtGui import QWidget, QColor, QBrush
-    from PyQt4.QtGui import QGridLayout
+    from PyQt4.QtGui import QGridLayout, QAction
     from PyQt4.QtGui import QComboBox, QSlider, QHBoxLayout, QLabel
     from PyQt4.QtGui import QVBoxLayout, QSpacerItem, QSizePolicy, QCheckBox
 except ImportError, e:
@@ -69,10 +70,12 @@ try:
     from background import Background
     from games import Games
     from statesaver import StateSaver
-    from scoringengine import HandContent, Meld
+    from hand import Hand
+    from meld import Meld
     from scoring import ExplainView, ScoringDialog, ScoreTable
-    from tables import TableList, SelectRuleset
-    from humanclient import HumanClient
+    from tables import SelectRuleset
+    from client import Client
+    from humanclient import HumanClient, AlreadyConnected, LoginAborted, NetworkOffline
     from rulesetselector import RulesetSelector
     from tilesetselector import TilesetSelector
     from backgroundselector import BackgroundSelector
@@ -81,6 +84,7 @@ try:
     from animation import animate, afterCurrentAnimationDo, Animated
     from player import Player, Players
     from game import Game, ScoringGame
+    from chat import ChatWindow
 
 except ImportError, e:
     NOTFOUND.append('kajongg is not correctly installed: modules: %s' % e)
@@ -109,6 +113,8 @@ class PlayConfigTab( QWidget):
         self.kcfg_rearrangeMelds.setObjectName('kcfg_rearrangeMelds')
         self.kcfg_showOnlyPossibleActions = QCheckBox(m18n('Show only possible actions'))
         self.kcfg_showOnlyPossibleActions.setObjectName('kcfg_showOnlyPossibleActions')
+        self.kcfg_propose = QCheckBox(m18n('Propose what to do'))
+        self.kcfg_propose.setObjectName('kcfg_propose')
         self.kcfg_animationSpeed = QSlider(self)
         self.kcfg_animationSpeed.setObjectName('kcfg_animationSpeed')
         self.kcfg_animationSpeed.setOrientation(Qt.Horizontal)
@@ -128,6 +134,7 @@ class PlayConfigTab( QWidget):
         vlayout.addWidget(self.kcfg_showShadows)
         vlayout.addWidget(self.kcfg_rearrangeMelds)
         vlayout.addWidget(self.kcfg_showOnlyPossibleActions)
+        vlayout.addWidget(self.kcfg_propose)
         vlayout.addWidget(self.kcfg_useSounds)
         vlayout.addWidget(self.kcfg_uploadVoice)
         vlayout.addLayout(sliderLayout)
@@ -177,7 +184,7 @@ class ConfigDialog(KConfigDialog):
             return
         KMessageBox.sorry(None, m18n('Cannot save your ruleset changes.<br>' \
             'You probably introduced a duplicate name. <br><br >Message from database:<br><br>' \
-           '<message>%1</message>', Query.lastError))
+           '<message>%1</message>', Query.dbhandle.lastError().text()))
 
     def reject(self):
         """rollback transaction"""
@@ -224,8 +231,7 @@ class SelectPlayers(SelectRuleset):
             self.grid.addWidget(cbName, idx+1, 1)
             self.nameWidgets.append(cbName)
             self.grid.addWidget(WindLabel(wind), idx+1, 0)
-            self.connect(cbName, SIGNAL('currentIndexChanged(int)'),
-                self.slotValidate)
+            cbName.currentIndexChanged.connect(self.slotValidate)
 
         query = Query("select p0,p1,p2,p3 from game where seed is null and game.id = (select max(id) from game)")
         if len(query.records):
@@ -255,17 +261,17 @@ class SelectPlayers(SelectRuleset):
         allNames = set(self.allNames)
         unusedNames = allNames - usedNames
         for combo in self.nameWidgets:
-            if not combo.manualSelect:
-                combo.setItemText(combo.currentIndex(), unusedNames.pop())
-        currentNames = set(unicode(x.currentText()) for x in self.nameWidgets)
-        for combo in self.nameWidgets:
-            comboName = unicode(combo.currentText())
             combo.blockSignals(True)
-            try:
+        try:
+            for combo in self.nameWidgets:
+                if combo.manualSelect:
+                    continue
+                comboName = unusedNames.pop()
                 combo.clear()
                 combo.addItems([comboName])
-                combo.addItems(sorted(allNames - currentNames))
-            finally:
+                combo.addItems(sorted(allNames - usedNames - set([comboName])))
+        finally:
+            for combo in self.nameWidgets:
                 combo.blockSignals(False)
         self.names = list(unicode(cbName.currentText()) for cbName in self.nameWidgets)
         assert len(set(self.names)) == 4
@@ -313,13 +319,18 @@ class VisiblePlayer(Player):
             color = Qt.black
         self.front.nameLabel.setBrush(QBrush(QColor(color)))
 
+    def getsFocus(self, dummyResults=None):
+        """give this player focus on his handBoard"""
+        self.handBoard.setEnabled(True)
+        self.handBoard.hasFocus = True
+
     def refreshManualRules(self, sender=None):
         """update status of manual rules"""
         assert InternalParameters.field
         if not self.handBoard:
             # might happen at program exit
             return
-        self.handContent = self.computeHandContent()
+        self.handContent = self.computeHand()
         currentScore = self.handContent.score
         hasManualScore = self.hasManualScore()
         for box in self.manualRuleBoxes:
@@ -338,47 +349,52 @@ class VisiblePlayer(Player):
                     if box != sender:
                         if applicable:
                             applicable = bool(box.rule.hasNonValueAction()) \
-                                or (self.computeHandContent(singleRule=box.rule).score > currentScore)
+                                or (self.computeHand(singleRule=box.rule).score > currentScore)
                 box.setApplicable(applicable)
 
     def __mjstring(self, singleRule):
         """compile hand info into a string as needed by the scoring engine"""
         winds = self.wind.lower() + 'eswn'[self.game.roundsFinished % 4]
-        wonChar = 'm'
         if self == self.game.winner:
             wonChar = 'M'
-        lastSource = 'd'
+        else:
+            wonChar = 'm'
         lastTile = InternalParameters.field.computeLastTile()
-        if len(lastTile) and lastTile[0].isupper():
+        if lastTile and lastTile.istitle():
             lastSource = 'w'
+        else:
+            lastSource = 'd'
         declaration = ''
         rules = [x.rule for x in self.manualRuleBoxes if x.isChecked()]
         if singleRule:
             rules.append(singleRule)
         for rule in rules:
-            actions = rule.actions
-            if'lastsource' in actions:
+            options = rule.options
+            if 'lastsource' in options:
                 if lastSource != '1':
                     # this defines precedences for source of last tile
-                    lastSource = actions['lastsource']
-            if 'declaration' in actions:
-                declaration = actions['declaration']
+                    lastSource = options['lastsource']
+            if 'declaration' in options:
+                declaration = options['declaration']
         return ''.join([wonChar, winds, lastSource, declaration])
 
     def __lastString(self):
         """compile hand info into a string as needed by the scoring engine"""
         if self != self.game.winner:
             return ''
-        return 'L%s%s' % (InternalParameters.field.computeLastTile(), InternalParameters.field.computeLastMeld().joined)
+        lastTile = InternalParameters.field.computeLastTile()
+        if not lastTile:
+            return ''
+        return 'L%s%s' % (lastTile, InternalParameters.field.computeLastMeld().joined)
 
-    def computeHandContent(self, withTile=None, robbedTile=None, singleRule=None):
-        """returns a HandContent object, using a cache"""
+    def computeHand(self, withTile=None, robbedTile=None, singleRule=None):
+        """returns a Hand object, using a cache"""
         game = self.game
         if not game.isScoringGame():
             # maybe we need a more extended class hierarchy for Player, VisiblePlayer, ScoringPlayer,
             # PlayingPlayer but not now. Just make sure that ExplainView can always call the
-            # same computeHandContent regardless of the game type
-            return Player.computeHandContent(self, withTile=withTile, robbedTile=robbedTile)
+            # same computeHand regardless of the game type
+            return Player.computeHand(self, withTile=withTile, robbedTile=robbedTile)
         if not self.handBoard:
             return None
         string = ' '.join([self.scoringString(), self.__mjstring(singleRule), self.__lastString()])
@@ -388,20 +404,26 @@ class VisiblePlayer(Player):
             cRules = []
         if singleRule:
             cRules.append(singleRule)
-        return HandContent.cached(game.ruleset, string,
-            computedRules=cRules)
+        return Hand.cached(self, string, computedRules=cRules)
 
     def popupMsg(self, msg):
         """shows a yellow message from player"""
         self.speak(msg.lower())
-        self.front.message.setText(m18nc('kajongg', msg))
-        self.front.message.setVisible(True)
+        yellow = self.front.message
+        yellow.setText('  '.join([unicode(yellow.msg), m18nc('kajongg', msg)]))
+        yellow.setVisible(True)
 
     def hidePopup(self):
         """hide the yellow message from player"""
         if isAlive(self.front.message):
+            self.front.message.msg = ''
             self.front.message.setVisible(False)
 
+    def speak(self, text):
+        """speak if we have a voice"""
+        if self.voice:
+            self.voice.speak(text, self.front.rotation())
+
 class PlayField(KXmlGuiWindow):
     """the main window"""
     # pylint: disable=R0902
@@ -411,6 +433,7 @@ class PlayField(KXmlGuiWindow):
         # see http://lists.kde.org/?l=kde-games-devel&m=120071267328984&w=2
         InternalParameters.field = self
         self.game = None
+        self.startingGame = False
         self.ignoreResizing = 1
         super(PlayField, self).__init__()
         self.background = None
@@ -421,12 +444,15 @@ class PlayField(KXmlGuiWindow):
         self.scoreTable = None
         self.explainView = None
         self.scoringDialog = None
-        self.tableLists = []
+        self.confDialog = None
         self.setupUi()
         KStandardAction.preferences(self.showSettings, self.actionCollection())
         self.applySettings()
         self.setupGUI()
         self.retranslateUi()
+        for action in self.toolBar().actions():
+            if 'onfigure' in action.text():
+                action.setPriority(QAction.LowPriority)
 
     def sizeHint(self):
         """give the main window a sensible default size"""
@@ -473,12 +499,12 @@ class PlayField(KXmlGuiWindow):
         if self.explainView:
             self.explainView.refresh(self.game)
 
-    def kajonggAction(self, name, icon, slot=None, shortcut=None, actionData=None):
+    def __kajonggAction(self, name, icon, slot=None, shortcut=None, actionData=None):
         """simplify defining actions"""
         res = KAction(self)
         res.setIcon(KIcon(icon))
         if slot:
-            self.connect(res, SIGNAL('triggered()'), slot)
+            res.triggered.connect(slot)
         self.actionCollection().addAction(name, res)
         if shortcut:
             res.setShortcut( Qt.CTRL + shortcut)
@@ -487,11 +513,11 @@ class PlayField(KXmlGuiWindow):
             res.setData(QVariant(actionData))
         return res
 
-    def kajonggToggleAction(self, name, icon, shortcut=None, actionData=None):
+    def __kajonggToggleAction(self, name, icon, shortcut=None, actionData=None):
         """a checkable action"""
-        res = self.kajonggAction(name, icon, shortcut=shortcut, actionData=actionData)
+        res = self.__kajonggAction(name, icon, shortcut=shortcut, actionData=actionData)
         res.setCheckable(True)
-        self.connect(res, SIGNAL('toggled(bool)'), self.__toggleWidget)
+        res.toggled.connect(self.__toggleWidget)
         return res
 
     def setupUi(self):
@@ -527,30 +553,35 @@ class PlayField(KXmlGuiWindow):
         self.centralView.setScene(scene)
         self.centralView.setFocusPolicy(Qt.StrongFocus)
         self.adjustView()
-        self.actionScoreGame = self.kajonggAction("scoreGame", "draw-freehand", self.scoreGame, Qt.Key_C)
-        self.actionPlayGame = self.kajonggAction("play", "arrow-right", self.playGame, Qt.Key_N)
-        self.actionAbortGame = self.kajonggAction("abort", "dialog-close", self.abortGame, Qt.Key_W)
+        self.actionScoreGame = self.__kajonggAction("scoreGame", "draw-freehand", self.scoreGame, Qt.Key_C)
+        self.actionPlayGame = self.__kajonggAction("play", "arrow-right", self.playGame, Qt.Key_N)
+        self.actionAbortGame = self.__kajonggAction("abort", "dialog-close", self.abort, Qt.Key_W)
         self.actionAbortGame.setEnabled(False)
-        self.actionQuit = self.kajonggAction("quit", "application-exit", self.quit, Qt.Key_Q)
-        self.actionPlayers = self.kajonggAction("players", "im-user", self.slotPlayers)
-        self.actionScoring = self.kajonggToggleAction("scoring", "draw-freehand",
+        self.actionQuit = self.__kajonggAction("quit", "application-exit", self.quit, Qt.Key_Q)
+        self.actionPlayers = self.__kajonggAction("players", "im-user", self.slotPlayers)
+        self.actionChat = self.__kajonggToggleAction("chat", "call-start",
+            shortcut=Qt.Key_H, actionData=ChatWindow)
+        game = self.game
+        self.actionChat.setEnabled(bool(game) and bool(game.client) and not game.client.hasLocalServer())
+        self.actionChat.setChecked(bool(game) and bool(game.client) and bool(game.client.table.chatWindow))
+        self.actionScoring = self.__kajonggToggleAction("scoring", "draw-freehand",
             shortcut=Qt.Key_S, actionData=ScoringDialog)
         self.actionScoring.setEnabled(False)
-        self.actionAngle = self.kajonggAction("angle", "object-rotate-left", self.changeAngle, Qt.Key_G)
+        self.actionAngle = self.__kajonggAction("angle", "object-rotate-left", self.changeAngle, Qt.Key_G)
         self.actionAngle.setEnabled(False)
         self.actionFullscreen = KToggleFullScreenAction(self.actionCollection())
         self.actionFullscreen.setShortcut(Qt.CTRL + Qt.Key_F)
         self.actionFullscreen.setShortcutContext(Qt.ApplicationShortcut)
         self.actionFullscreen.setWindow(self)
         self.actionCollection().addAction("fullscreen", self.actionFullscreen)
-        self.connect(self.actionFullscreen, SIGNAL('toggled(bool)'), self.fullScreen)
-        self.actionScoreTable = self.kajonggToggleAction("scoreTable", "format-list-ordered",
+        self.actionFullscreen.toggled.connect(self.fullScreen)
+        self.actionScoreTable = self.__kajonggToggleAction("scoreTable", "format-list-ordered",
             Qt.Key_T, actionData=ScoreTable)
-        self.actionExplain = self.kajonggToggleAction("explain", "applications-education",
+        self.actionExplain = self.__kajonggToggleAction("explain", "applications-education",
             Qt.Key_E, actionData=ExplainView)
-        self.actionAutoPlay = self.kajonggAction("demoMode", "arrow-right", None, Qt.Key_D)
+        self.actionAutoPlay = self.__kajonggAction("demoMode", "arrow-right-double", None, Qt.Key_D)
         self.actionAutoPlay.setCheckable(True)
-        self.connect(self.actionAutoPlay, SIGNAL('toggled(bool)'), self.__toggleDemoMode)
+        self.actionAutoPlay.toggled.connect(self.__toggleDemoMode)
         self.actionAutoPlay.setChecked(InternalParameters.autoPlay)
         QMetaObject.connectSlotsByName(self)
 
@@ -570,21 +601,57 @@ class PlayField(KXmlGuiWindow):
         """toggle between full screen and normal view"""
         self.actionFullscreen.setFullScreen(self, toggle)
 
+    def confirmAbort(self):
+        """optionally ask if he really wants to abort the game"""
+        if (self.game is None
+                or self.game.finished()):
+            return True
+        msg = m18n("Do you really want to abort this game?")
+        return KMessageBox.questionYesNo (None, msg) == KMessageBox.Yes
+
+    def abort(self):
+        """abort current game"""
+        if not self.game:
+            self.startingGame = False
+            return succeed(None)
+        demoMode = self.actionAutoPlay.isChecked()
+        self.actionAutoPlay.setChecked(False)
+        if not self.confirmAbort():
+            self.actionAutoPlay.setChecked(demoMode)
+            return
+        return self.abortGame()
+
     def quit(self):
         """exit the application"""
-        return self.abortGame(HumanClient.gameClosed)
+        deferred = self.abort()
+        if deferred:
+            deferred.addCallback(Client.shutdownClients).addCallback(Client.quitProgram)
+        return deferred
 
-    def abortGame(self, callback=None):
+    def hideGame(self, dummyResult=None):
+        """remove all visible traces of the current game"""
+        self.setWindowTitle('Kajongg')
+        self.discardBoard.hide()
+        self.selectorBoard.tiles = []
+        self.selectorBoard.allSelectorTiles = []
+        self.centralScene.removeTiles()
+        if self.clientDialog:
+            self.clientDialog.hide()
+            self.clientDialog = None
+        if self.game:
+            self.game.removeGameFromPlayfield()
+            self.game = None
+        self.updateGUI()
+
+    def abortGame(self):
         """if a game is active, abort it"""
         self.actionAutoPlay.setChecked(False)
-        if self.game:
-            if self.game.client:
-                return self.game.client.abortGame(callback)
-            else:
-                self.game.close()
-        if callback:
-            callback()
-        return True
+        self.startingGame = False
+        if self.game.isScoringGame():
+            self.hideGame()
+            return succeed(None)
+        else:
+            return self.game.close()
 
     def closeEvent(self, event):
         """somebody wants us to close, maybe ALT-F4 or so"""
@@ -598,12 +665,18 @@ class PlayField(KXmlGuiWindow):
         assert self.game.isScoringGame()
         assert isinstance(tile, Tile), (tile, str(tile))
         currentBoard = tile.board
-        tile, meld = currentBoard.dragObject(tile)
+        dragTile, dragMeld = currentBoard.dragObject(tile)
         if wind == 'X':
             receiver = self.selectorBoard
         else:
             receiver = self.game.players[wind].handBoard
-        receiver.dropHere(tile, meld, lowerHalf)
+        if receiver != currentBoard or bool(lowerHalf) != bool(tile.yoffset):
+            movingLastMeld = tile.element in self.computeLastMeld().pairs
+            if movingLastMeld:
+                self.scoringDialog.clearLastTileCombo()
+            receiver.dropHere(dragTile, dragMeld, lowerHalf)
+            if movingLastMeld and receiver == currentBoard:
+                self.scoringDialog.fillLastTileCombo()
 
     def __navigateScoringGame(self, event):
         """keyboard navigation in a scoring game"""
@@ -642,16 +715,49 @@ class PlayField(KXmlGuiWindow):
 
     def retranslateUi(self):
         """retranslate"""
-        self.actionScoreGame.setText(m18n("&Score Manual Game"))
-        self.actionPlayGame.setText(m18n("&Play"))
-        self.actionAbortGame.setText(m18n("&Abort"))
-        self.actionQuit.setText(m18n("&Quit"))
-        self.actionPlayers.setText(m18n("&Players"))
-        self.actionAngle.setText(m18n("&Change Visual Angle"))
-        self.actionScoring.setText(m18n("&Scoring"))
-        self.actionScoreTable.setText(m18nc('kajongg', "&Score Table"))
-        self.actionExplain.setText(m18n("&Explain Scores"))
-        self.actionAutoPlay.setText(m18n("&Demo Mode"))
+        self.actionScoreGame.setText(m18nc('@action:inmenu', "&Score Manual Game"))
+        self.actionScoreGame.setIconText(m18nc('@action:intoolbar', 'Manual Game'))
+        self.actionScoreGame.setHelpText(m18nc('@info:tooltip', '&Score a manual game.'))
+
+        self.actionPlayGame.setText(m18nc('@action:intoolbar', "&Play"))
+        self.actionPlayGame.setPriority(QAction.LowPriority)
+        self.actionPlayGame.setHelpText(m18nc('@info:tooltip', 'Start a new game.'))
+
+        self.actionAbortGame.setText(m18nc('@action:inmenu', "&Abort Game"))
+        self.actionAbortGame.setPriority(QAction.LowPriority)
+        self.actionAbortGame.setHelpText(m18nc('@info:tooltip', 'Abort the current game.'))
+
+        self.actionQuit.setText(m18nc('@action:inmenu', "&Quit Kajongg"))
+        self.actionQuit.setPriority(QAction.LowPriority)
+
+        self.actionPlayers.setText(m18nc('@action:intoolbar', "&Players"))
+        self.actionPlayers.setHelpText(m18nc('@info:tooltip', 'define your players.'))
+
+        self.actionAngle.setText(m18nc('@action:inmenu', "&Change Visual Angle"))
+        self.actionAngle.setIconText(m18nc('@action:intoolbar', "Angle"))
+        self.actionAngle.setHelpText(m18nc('@info:tooltip', "Change the visual appearance of the tiles."))
+
+        self.actionScoring.setText(m18nc('@action:inmenu', "&Show Scoring Editor"))
+        self.actionScoring.setIconText(m18nc('@action:intoolbar', "&Scoring"))
+        self.actionScoring.setHelpText(m18nc('@info:tooltip', "Show or hide the scoring editor for a manual game."))
+
+        self.actionScoreTable.setText(m18nc('kajongg @action:inmenu', "&Score Table"))
+        self.actionScoreTable.setIconText(m18nc('kajongg @action:intoolbar', "&Scores"))
+        self.actionScoreTable.setHelpText(m18nc('kajongg @info:tooltip',
+                "Show or hide the score table for the current game."))
+
+        self.actionExplain.setText(m18nc('@action:inmenu', "&Explain Scores"))
+        self.actionExplain.setIconText(m18nc('@action:intoolbar', "&Explain"))
+        self.actionExplain.setHelpText(m18nc('@info:tooltip',
+                'Explain the scoring for all players in the current game.'))
+
+        self.actionAutoPlay.setText(m18nc('@action:inmenu', "&Demo Mode"))
+        self.actionAutoPlay.setPriority(QAction.LowPriority)
+        self.actionAutoPlay.setHelpText(m18nc('@info:tooltip',
+                'Let the computer take over for you. Start a new local game if needed.'))
+
+        self.actionChat.setText(m18n("C&hat"))
+        self.actionChat.setHelpText(m18nc('@info:tooltip', 'Chat with the other players.'))
 
     def changeEvent(self, event):
         """when the applicationwide language changes, recreate GUI"""
@@ -680,7 +786,7 @@ class PlayField(KXmlGuiWindow):
             if self.game:
                 self.game.throwDices()
         gameSelector.close()
-        self.refresh()
+        self.updateGUI()
         return bool(self.game)
 
     def scoreGame(self):
@@ -690,7 +796,16 @@ class PlayField(KXmlGuiWindow):
 
     def playGame(self):
         """play a remote game: log into a server and show its tables"""
-        self.tableLists.append(TableList())
+        self.startingGame = True
+        self.updateGUI()
+        try:
+            HumanClient()
+        except AlreadyConnected:
+            pass
+        except LoginAborted:
+            pass
+        except NetworkOffline, exception:
+            KMessageBox.sorry(None, m18n('You have no network connection (error code %1).', str(exception)))
 
     def adjustView(self):
         """adjust the view such that exactly the wanted things are displayed
@@ -788,10 +903,9 @@ class PlayField(KXmlGuiWindow):
 
     def __showSettings2(self, dummyResult):
         """now that no animation is running, show settings dialog"""
-        confDialog = ConfigDialog(self, "settings")
-        self.connect(confDialog, SIGNAL('settingsChanged(QString)'),
-           self.applySettings)
-        confDialog.show()
+        self.confDialog = ConfigDialog(self, "settings")
+        self.confDialog.settingsChanged.connect(self.applySettings)
+        self.confDialog.show()
 
     def newGame(self):
         """asks user for players and ruleset for a new game and returns that new game"""
@@ -807,12 +921,15 @@ class PlayField(KXmlGuiWindow):
         actionData = action.data().toPyObject()
         if checked:
             if isinstance(actionData, type):
-                actionData = actionData(self.game)
+                if actionData == ChatWindow:
+                    actionData = ChatWindow.createFor(self.game.client.table)
+                else:
+                    actionData = actionData(self.game)
                 action.setData(QVariant(actionData))
                 if isinstance(actionData, ScoringDialog):
                     self.scoringDialog = actionData
-                    self.connect(actionData.btnSave, SIGNAL('clicked(bool)'), self.nextHand)
-                    self.connect(actionData, SIGNAL('scoringClosed()'), self.scoringClosed)
+                    actionData.btnSave.clicked.connect(self.nextScoringHand)
+                    actionData.scoringClosed.connect(self.__scoringClosed)
                 elif isinstance(actionData, ExplainView):
                     self.explainView = actionData
                 elif isinstance(actionData, ScoreTable):
@@ -833,13 +950,14 @@ class PlayField(KXmlGuiWindow):
         else:
             InternalParameters.autoPlay = checked
             if checked:
+                # TODO: use the last used ruleset. Right now it always takes the first of the list.
                 self.playGame()
 
-    def scoringClosed(self):
+    def __scoringClosed(self):
         """the scoring window has been closed with ALT-F4 or similar"""
         self.actionScoring.setChecked(False)
 
-    def nextHand(self):
+    def nextScoringHand(self):
         """save hand to database, update score table and balance in status line, prepare next hand"""
         if self.game.winner:
             for player in self.game.players:
@@ -854,13 +972,11 @@ class PlayField(KXmlGuiWindow):
 
     def prepareHand(self):
         """redecorate wall"""
-        if not self.game:
-            self.refresh()
-            return
-        self.refresh()
-        self.game.wall.decorate()
+        self.updateGUI()
+        if self.game:
+            self.game.wall.decorate()
 
-    def refresh(self):
+    def updateGUI(self):
         """update some actions, all auxiliary windows and the statusbar"""
         game = self.game
         for action in [self.actionScoreGame, self.actionPlayGame]:
@@ -872,6 +988,11 @@ class PlayField(KXmlGuiWindow):
         self.selectorBoard.setEnabled(scoring)
         self.discardBoard.setVisible(bool(game) and not scoring)
         self.actionScoring.setEnabled(scoring and not game.finished())
+        self.actionAutoPlay.setEnabled(not self.startingGame and not scoring)
+        self.actionChat.setEnabled(bool(game) and bool(game.client)
+            and not game.client.hasLocalServer() and not self.startingGame)
+            # chatting on tables before game started works with chat button per table
+        self.actionChat.setChecked(self.actionChat.isEnabled() and bool(game.client.table.chatWindow))
         if self.actionScoring.isChecked():
             self.actionScoring.setChecked(scoring and not game.finished())
         for view in [self.scoringDialog, self.explainView, self.scoreTable]:
@@ -920,7 +1041,6 @@ class PlayField(KXmlGuiWindow):
         """compile hand info into a string as needed by the scoring engine"""
         if self.scoringDialog:
             return self.scoringDialog.computeLastTile()
-        return ''
 
     def computeLastMeld(self):
         """compile hand info into a string as needed by the scoring engine"""
