@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2012 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@ along with this program if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
-from PyQt4.QtCore import Qt, QPointF, QVariant, SIGNAL, SLOT, \
+from PyQt4.QtCore import Qt, QPointF, QVariant, SLOT, pyqtSignal, \
     QSize, QModelIndex, QEvent, QTimer
 
 from PyQt4.QtGui import QColor, QPushButton, QPixmapCache
@@ -41,7 +41,6 @@ from util import m18n, m18nc
 from common import WINDS, InternalParameters, Debug
 from statesaver import StateSaver
 from query import Query
-from scoringengine import Score
 from guiutil import ListComboBox
 from tree import TreeItem, RootItem, TreeModel
 
@@ -234,7 +233,7 @@ class ScoreModel(TreeModel):
                 hands = child1.hands()
                 handResult = hands[section-1]
                 if not handResult.penalty:
-                    return '%s/%d' % (handResult.prevailing, handResult.roundHand(hands))
+                    return handResult.handId()
         elif role == Qt.TextAlignmentRole:
             if section == 0:
                 return QVariant(int(Qt.AlignLeft|Qt.AlignVCenter))
@@ -246,13 +245,14 @@ class ScoreModel(TreeModel):
         """loads all data from the data base into a 2D matrix formatted like the wanted tree"""
         game = self.scoreTable.game
         data = []
-        for idx, player in enumerate(game.players):
-            records = Query('select rotated,penalty,won,prevailing,wind,points,payments,balance,manualrules'
-                        ' from score where game=? and player=? order by hand',
-                        list([game.gameid, player.nameid])).records
-            playerTuple = tuple([player.name, [HandResult(*x) for x in records]]) # pylint: disable=W0142
-            # pylint * magic
-            data.append(playerTuple)
+        records = Query(
+                'select player,rotated,notrotated,penalty,won,prevailing,wind,points,payments,balance,manualrules'
+                ' from score where game=? order by player,hand',
+                list([game.gameid])).records
+        # pylint: disable=W0142
+        # pylint * magic
+        data = list(tuple([m18nc('kajongg', player.name), [HandResult(*x[1:]) for x in records \
+                if x[0] == player.nameid]]) for player in game.players)
         self.__findMinMaxChartPoints(data)
         parent = QModelIndex()
         groupIndex = self.index(self.rootItem.childCount(), 0, parent)
@@ -282,8 +282,9 @@ class HandResult(object):
     """holds the results of a hand for the scoring table"""
     # pylint: disable=R0913
     # we have too many arguments
-    def __init__(self, rotated, penalty, won, prevailing, wind, points, payments, balance, manualrules):
+    def __init__(self, rotated, notRotated, penalty, won, prevailing, wind, points, payments, balance, manualrules):
         self.rotated = rotated
+        self.notRotated = notRotated
         self.penalty = bool(penalty)
         self.won = won
         self.prevailing = prevailing
@@ -297,6 +298,11 @@ class HandResult(object):
         return '%d %d %s %d %d %s' % (
                 self.penalty, self.points, self.wind, self.payments, self.balance, self.manualrules)
 
+    def handId(self):
+        """identifies the hand for window title and scoring table"""
+        character = chr(ord('a') - 1 + self.notRotated) if self.notRotated else ''
+        return '%s%s%s' % (self.prevailing, self.rotated + 1, character)
+
     def roundHand(self, allHands):
         """the nth hand in the current round, starting with 1"""
         idx = allHands.index(self)
@@ -487,10 +493,9 @@ class ScoreTable(QWidget):
         self.viewRight.setFrameStyle(QFrame.NoFrame)
         self.viewLeft.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         for master, slave in ((self.viewRight, self.viewLeft), (self.viewLeft, self.viewRight)):
-            self.connect(master, SIGNAL('expanded(const QModelIndex&)'), slave.expand)
-            self.connect(master, SIGNAL('collapsed(const QModelIndex&)'), slave.collapse)
-            self.connect(master.verticalScrollBar(), SIGNAL('valueChanged(int)'),
-                slave.verticalScrollBar().setValue)
+            master.expanded.connect(slave.expand)
+            master.collapsed.connect(slave.collapse)
+            master.verticalScrollBar().valueChanged.connect(slave.verticalScrollBar().setValue)
         for row, expand in enumerate(expandGroups):
             self.viewLeft.setExpanded(self.scoreModel.index(row, 0, QModelIndex()), expand)
         self.viewLeft.resizeColumnToContents(0)
@@ -547,13 +552,11 @@ class ExplainView(QListView):
             for player in self.game.players:
                 iName = m18nc('kajongg', player.name)
                 pLines = []
-                if player.handContent and player.handContent.tiles:
-                    score = player.handContent.score
+                if player.handContent and player.handContent.tileNames:
                     total = player.handContent.total()
-                    pLines = player.handContent.explain()
-                    pLines = [m18n('Computed scoring for %1:', iName)] + pLines
-                    pLines.append(m18n('Total for %1: %2 base points, %3 doubles, %4 points',
-                        iName, score.points, score.doubles, total))
+                    pLines = ['%s: %s' % (iName, total)]
+                    for line in player.handContent.explain():
+                        pLines.append('- ' + line)
                 elif player.handTotal:
                     pLines.append(m18n('Manual score for %1: %2 points', iName, player.handTotal))
                 pLines.append('')
@@ -565,15 +568,32 @@ class PenaltyBox(QSpinBox):
     def __init__(self, parties, parent=None):
         QSpinBox.__init__(self, parent)
         self.parties = parties
+        self.prevValue = None
 
     def validate(self, inputData, pos):
-        """ensure the value is a multiple of parties"""
+        """check if value is a multiple of parties"""
         result, newPos = QSpinBox.validate(self, inputData, pos)
         if result == QValidator.Acceptable:
             if int(inputData) % self.parties != 0:
                 result = QValidator.Intermediate
+        if result == QValidator.Acceptable:
+            self.prevValue = str(inputData)
         return (result, newPos)
 
+    def fixup(self, data):
+        """change input to a legal value"""
+        value = int(str(data))
+        prevValue = int(str(self.prevValue))
+        assert value != prevValue
+        common = int(self.parties * 10)
+        small = value // common
+        if value > prevValue:
+            newV = str(int((small + 1) * common))
+        else:
+            newV = str(int(small * common))
+        data.clear()
+        data.append(newV)
+
 class RuleBox(QCheckBox):
     """additional attribute: ruleId"""
     def __init__(self, rule):
@@ -595,7 +615,7 @@ class PenaltyDialog(QDialog):
         self.game = game
         grid = QGridLayout(self)
         lblOffense = QLabel(m18n('Offense:'))
-        crimes = list([x for x in game.ruleset.penaltyRules if not ('absolute' in x.actions and game.winner)])
+        crimes = list([x for x in game.ruleset.penaltyRules if not ('absolute' in x.options and game.winner)])
         self.cbCrime = ListComboBox(crimes)
         lblOffense.setBuddy(self.cbCrime)
         grid.addWidget(lblOffense, 0, 0)
@@ -625,13 +645,13 @@ class PenaltyDialog(QDialog):
         grid.addWidget(QLabel(''), 6, 0)
         grid.setRowStretch(6, 10)
         for player in self.payers + self.payees:
-            self.connect(player, SIGNAL('currentIndexChanged(int)'), self.playerChanged)
-        self.connect(self.spPenalty, SIGNAL('valueChanged(int)'), self.penaltyChanged)
-        self.connect(self.cbCrime, SIGNAL('currentIndexChanged(int)'), self.crimeChanged)
+            player.currentIndexChanged.connect(self.playerChanged)
+        self.spPenalty.valueChanged.connect(self.penaltyChanged)
+        self.cbCrime.currentIndexChanged.connect(self.crimeChanged)
         buttonBox = KDialogButtonBox(self)
         grid.addWidget(buttonBox, 7, 0, 1, 5)
         buttonBox.setStandardButtons(QDialogButtonBox.Cancel)
-        self.connect(buttonBox, SIGNAL("rejected()"), self, SLOT("reject()"))
+        buttonBox.rejected.connect(self.reject)
         self.btnExecute = buttonBox.addButton(m18n("&Execute"), QDialogButtonBox.AcceptRole,
             self, SLOT("accept()"))
         self.crimeChanged()
@@ -679,48 +699,45 @@ class PenaltyDialog(QDialog):
     def crimeChanged(self):
         """another offense has been selected"""
         offense = self.cbCrime.current
-        payers = int(offense.actions.get('payers', 1))
-        payees = int(offense.actions.get('payees', 1))
-        self.spPenalty.setValue(-offense.score.value)
+        payers = int(offense.options.get('payers', 1))
+        payees = int(offense.options.get('payees', 1))
+        self.spPenalty.prevValue = str(-offense.score.points)
+        self.spPenalty.setValue(-offense.score.points)
         self.spPenalty.parties = max(payers, payees)
         self.spPenalty.setSingleStep(10 )
-        self.lblUnits.setText(Score.unitName(offense.score.unit))
-        for pList, count in ((self.payers, payers), (self.payees, payees)):
-            for idx, payer in enumerate(pList):
-                payer.setVisible(idx<count)
-                payer.lblPayment.setVisible(idx<count)
-                if idx < count:
-                    payString = '%d %s' % (
-                        -offense.score.value//count, Score.unitName(offense.score.unit))
-                    if pList == self.payers:
-                        payer.lblPayment.setText(m18nc('penalty dialog, appears behind paying player combobox',
-                            'pays %1', payString))
-                    else:
-                        payer.lblPayment.setText(m18nc('penalty dialog, appears behind profiting player combobox',
-                            'gets %1', payString))
+        self.lblUnits.setText(m18n('points'))
         self.playerChanged()
+        self.penaltyChanged()
 
     def penaltyChanged(self):
         """total has changed, update payments"""
+        # normally value is only validated when leaving the field
+        self.spPenalty.interpretText()
         offense = self.cbCrime.current
         penalty = self.spPenalty.value()
-        payers = int(offense.actions.get('payers', 1))
-        payees = int(offense.actions.get('payees', 1))
+        payers = int(offense.options.get('payers', 1))
+        payees = int(offense.options.get('payees', 1))
         payerAmount = -penalty // payers
         payeeAmount = penalty // payees
-        for pList, amount in [(self.payers, payerAmount), (self.payees, payeeAmount)]:
-            for player in pList:
-                if player.isVisible():
-                    player.lblPayment.setText('%d %s' % (
-                        amount, Score.unitName(offense.score.unit)))
-                else:
-                    player.lblPayment.setText('')
+        for pList, amount, count in ((self.payers, payerAmount, payers), (self.payees, payeeAmount, payees)):
+            for idx, player in enumerate(pList):
+                player.setVisible(idx<count)
+                player.lblPayment.setVisible(idx<count)
+                if idx < count:
+                    if pList == self.payers:
+                        player.lblPayment.setText(m18nc('penalty dialog, appears behind paying player combobox',
+                            'pays %1 points', -amount))
+                    else:
+                        player.lblPayment.setText(m18nc('penalty dialog, appears behind profiting player combobox',
+                            'gets %1 points', amount))
 
 class ScoringDialog(QWidget):
     """a dialog for entering the scores"""
     # pylint: disable=R0902
     # pylint we need more than 10 instance attributes
 
+    scoringClosed = pyqtSignal()
+
     def __init__(self, game):
         QWidget.__init__(self, None)
         self.game = None
@@ -746,18 +763,16 @@ class ScoringDialog(QWidget):
         for idx in range(4):
             self.setupUiForPlayer(pGrid, idx)
         self.draw = QCheckBox(m18nc('kajongg','Draw'))
-        self.connect(self.draw, SIGNAL('clicked(bool)'), self.wonChanged)
+        self.draw.clicked.connect(self.wonChanged)
         btnPenalties = QPushButton(m18n("&Penalties"))
-        self.connect(btnPenalties, SIGNAL('clicked(bool)'), self.penalty)
+        btnPenalties.clicked.connect(self.penalty)
         self.btnSave = QPushButton(m18n('&Save Hand'))
         self.btnSave.setEnabled(False)
         self.setupUILastTileMeld(pGrid)
         pGrid.setRowStretch(87, 10)
         pGrid.addWidget(self.draw, 7, 3)
-        self.connect(self.cbLastTile, SIGNAL('currentIndexChanged(int)'),
-            self.slotLastTile)
-        self.connect(self.cbLastMeld, SIGNAL('currentIndexChanged(int)'),
-            self.slotInputChanged)
+        self.cbLastTile.currentIndexChanged.connect(self.slotLastTile)
+        self.cbLastMeld.currentIndexChanged.connect(self.slotInputChanged)
         btnBox = QHBoxLayout()
         btnBox.addWidget(btnPenalties)
         btnBox.addWidget(self.btnSave)
@@ -800,8 +815,8 @@ class ScoringDialog(QWidget):
         pGrid.addWidget(self.spValues[idx], idx+2, 2)
         self.wonBoxes[idx] = QCheckBox("")
         pGrid.addWidget(self.wonBoxes[idx], idx+2, 3)
-        self.connect(self.wonBoxes[idx], SIGNAL('clicked(bool)'), self.wonChanged)
-        self.connect(self.spValues[idx], SIGNAL('valueChanged(int)'), self.slotInputChanged)
+        self.wonBoxes[idx].clicked.connect(self.wonChanged)
+        self.spValues[idx].valueChanged.connect(self.slotInputChanged)
         detailTab = QWidget()
         self.detailTabs.addTab(detailTab,'')
         self.details[idx] = QWidget()
@@ -826,16 +841,15 @@ class ScoringDialog(QWidget):
                         self.detailsLayout[idx].removeWidget(child)
                         del child
                 if game:
-                    self.spValues[idx].setRange(0, game.ruleset.limit)
+                    self.spValues[idx].setRange(0, game.ruleset.limit or 99999)
                     self.nameLabels[idx].setText(m18nc('kajongg', player.name))
                     self.windLabels[idx].wind = player.wind
                     self.windLabels[idx].roundsFinished = game.roundsFinished
                     self.detailTabs.setTabText(idx, m18nc('kajongg', player.name))
-                    player.manualRuleBoxes = [RuleBox(x) for x in game.ruleset.allRules.values() if x.manualRegex]
+                    player.manualRuleBoxes = [RuleBox(x) for x in game.ruleset.allRules if x.hasSelectable]
                     for ruleBox in player.manualRuleBoxes:
                         self.detailsLayout[idx].addWidget(ruleBox)
-                        self.connect(ruleBox, SIGNAL('clicked(bool)'),
-                            self.slotInputChanged)
+                        ruleBox.clicked.connect(self.slotInputChanged)
                 player.refreshManualRules()
 
     def show(self):
@@ -852,6 +866,8 @@ class ScoringDialog(QWidget):
     def slotLastTile(self):
         """called when the last tile changes"""
         newLastTile = self.computeLastTile()
+        if not newLastTile:
+            return
         prevLower, newLower = self.prevLastTile.islower(), newLastTile.islower()
         if prevLower != newLower:
             # state of last tile (concealed/exposed) changed:
@@ -861,7 +877,7 @@ class ScoringDialog(QWidget):
                 for box in winner.manualRuleBoxes:
                     if box.isChecked():
                         box.setChecked(False)
-                        hand = winner.computeHandContent()
+                        hand = winner.computeHand()
                         if hand.manualRuleMayApply(box.rule):
                             box.setChecked(True)
         self.prevLastTile = newLastTile
@@ -872,13 +888,12 @@ class ScoringDialog(QWidget):
         idx = self.cbLastTile.currentIndex()
         if idx >= 0:
             return str(self.cbLastTile.itemData(idx).toString())
-        return ''
 
     def closeEvent(self, event):
         """the user pressed ALT-F4"""
         self.hide()
         event.ignore()
-        self.emit(SIGNAL('scoringClosed()'))
+        self.scoringClosed.emit()
 
     def clickedPlayerIdx(self, checkbox):
         """the player whose box has been clicked"""
@@ -912,8 +927,8 @@ class ScoringDialog(QWidget):
         if isinstance(self.sender(), RuleBox):
             ruleBox = self.sender()
             if ruleBox.isChecked() and ruleBox.rule.exclusive():
-                for player in self.game.players:
-                    if ruleBox.parentWidget() != self.details[player.idx]:
+                for idx, player in enumerate(self.game.players):
+                    if ruleBox.parentWidget() != self.details[idx]:
                         for pBox in player.manualRuleBoxes:
                             if pBox.rule.name == ruleBox.rule.name:
                                 pBox.setChecked(False)
@@ -955,6 +970,8 @@ class ScoringDialog(QWidget):
 
     def computeScores(self):
         """if tiles have been selected, compute their value"""
+        # pylint: disable=R0912
+        # too many branches
         if not self.game:
             return
         if self.game.finished():
@@ -965,10 +982,16 @@ class ScoringDialog(QWidget):
             self.wonBoxes[idx].blockSignals(True) # we do not want that change to call computeScores again
             if player.handBoard and player.handBoard.tiles:
                 self.spValues[idx].setEnabled(False)
+                self.nameLabels[idx].setBuddy(self.wonBoxes[idx])
                 for loop in range(10):
                     prevTotal = player.handTotal
-                    player.handContent = player.computeHandContent()
-                    self.wonBoxes[idx].setVisible(player.handContent.maybeMahjongg())
+                    oldWinner = self.game.winner
+                    try:
+                        self.game.winner = player
+                        player.handContent = player.computeHand()
+                    finally:
+                        self.game.winner = oldWinner
+                    self.wonBoxes[idx].setVisible(player.handContent.won)
                     if not self.wonBoxes[idx].isVisibleTo(self) and self.wonBoxes[idx].isChecked():
                         self.wonBoxes[idx].setChecked(False)
                         self.game.winner = None
@@ -977,12 +1000,13 @@ class ScoringDialog(QWidget):
                     player.refreshManualRules()
                 self.spValues[idx].setValue(player.handTotal)
             else:
-                player.handContent = player.computeHandContent()
+                player.handContent = player.computeHand()
                 if not self.spValues[idx].isEnabled():
                     self.spValues[idx].clear()
                     self.spValues[idx].setValue(0)
                     self.spValues[idx].setEnabled(True)
-                self.wonBoxes[idx].setVisible(player.handTotal >= self.game.ruleset.minMJTotal)
+                    self.nameLabels[idx].setBuddy(self.spValues[idx])
+                self.wonBoxes[idx].setVisible(player.handTotal >= self.game.ruleset.minMJTotal())
                 if not self.wonBoxes[idx].isVisibleTo(self) and self.wonBoxes[idx].isChecked():
                     self.wonBoxes[idx].setChecked(False)
             if not self.wonBoxes[idx].isVisibleTo(self) and player is self.game.winner:
@@ -999,7 +1023,7 @@ class ScoringDialog(QWidget):
         if self.game.winner and self.game.winner.handBoard:
             winnerTiles = self.game.winner.handBoard.tiles
             pairs = []
-            for meld in self.game.winner.computeHandContent().melds:
+            for meld in self.game.winner.computeHand().melds:
                 if len(meld) < 4:
                     pairs.extend(meld.pairs)
             for tile in winnerTiles:
@@ -1043,6 +1067,11 @@ class ScoringDialog(QWidget):
         self.cbLastTile.setCurrentIndex(restoredIdx)
         self.prevLastTile = self.computeLastTile()
 
+    def clearLastTileCombo(self):
+        """as the name says"""
+        self.comboTilePairs = None
+        self.cbLastTile.clear()
+
     def fillLastTileCombo(self):
         """fill the drop down list with all possible tiles.
         If the drop down had content before try to preserve the
@@ -1057,7 +1086,7 @@ class ScoringDialog(QWidget):
             self.__fillLastTileComboWith(lastTiles, winnerTiles)
         finally:
             self.cbLastTile.blockSignals(False)
-            self.cbLastTile.emit(SIGNAL("currentIndexChanged(int)"), 0)
+            self.cbLastTile.currentIndexChanged.emit(0)
 
     def __fillLastMeldComboWith(self, winnerMelds, indexedMeld, lastTile):
         """fill last meld combo with prepared content"""
@@ -1119,7 +1148,7 @@ class ScoringDialog(QWidget):
             if self.cbLastTile.count() == 0:
                 return
             lastTile = InternalParameters.field.computeLastTile()
-            winnerMelds = [m for m in self.game.winner.computeHandContent().melds if len(m) < 4 \
+            winnerMelds = [m for m in self.game.winner.computeHand().melds if len(m) < 4 \
                 and lastTile in m.pairs]
             assert len(winnerMelds)
             if len(winnerMelds) == 1:
@@ -1132,7 +1161,7 @@ class ScoringDialog(QWidget):
             self.lblLastMeld.setVisible(showCombo)
             self.cbLastMeld.setVisible(showCombo)
             self.cbLastMeld.blockSignals(False)
-            self.cbLastMeld.emit(SIGNAL("currentIndexChanged(int)"), 0)
+            self.cbLastMeld.currentIndexChanged.emit(0)
 
     def slotInputChanged(self):
         """some input fields changed: update"""
@@ -1145,7 +1174,7 @@ class ScoringDialog(QWidget):
         game = self.game
         if game:
             valid = True
-            if game.winner and game.winner.handTotal < game.ruleset.minMJTotal:
+            if game.winner and game.winner.handTotal < game.ruleset.minMJTotal():
                 valid = False
             elif not game.winner and not self.draw.isChecked():
                 valid = False
